@@ -1,7 +1,7 @@
 /*
  * FeatureWorkerManager.cpp - class for managing feature worker instances
  *
- * Copyright (c) 2017-2020 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2017-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -37,12 +37,10 @@
 
 // clazy:excludeall=detaching-member
 
-FeatureWorkerManager::FeatureWorkerManager( VeyonServerInterface& server, FeatureManager& featureManager, QObject* parent ) :
+FeatureWorkerManager::FeatureWorkerManager( VeyonServerInterface& server, QObject* parent ) :
 	QObject( parent ),
 	m_server( server ),
-	m_featureManager( featureManager ),
-	m_tcpServer( this ),
-	m_workersMutex( QMutex::Recursive )
+	m_tcpServer( this )
 {
 	connect( &m_tcpServer, &QTcpServer::newConnection,
 			 this, &FeatureWorkerManager::acceptConnection );
@@ -92,8 +90,20 @@ bool FeatureWorkerManager::startManagedSystemWorker( Feature::Uid featureUid )
 	connect( worker.process, static_cast<void(QProcess::*)(int, QProcess::ExitStatus)>(&QProcess::finished),
 			 worker.process, &QProcess::deleteLater );
 
-	vDebug() << "Starting managed system worker for feature" << featureUid;
-	worker.process->start( VeyonCore::filesystem().workerFilePath(), { featureUid.toString() } );
+	vDebug() << "Starting managed system worker for feature" << VeyonCore::featureManager().feature(featureUid).name();
+	if( qEnvironmentVariableIsSet("VEYON_VALGRIND_WORKERS") )
+	{
+		worker.process->start( QStringLiteral("valgrind"),
+							   { QStringLiteral("--error-limit=no"),
+								 QStringLiteral("--leak-check=full"),
+								 QStringLiteral("--show-leak-kinds=all"),
+								 QStringLiteral("--log-file=valgrind-%1.log").arg(VeyonCore::formattedUuid(featureUid)),
+								 VeyonCore::filesystem().workerFilePath(), featureUid.toString() } );
+	}
+	else
+	{
+		worker.process->start( VeyonCore::filesystem().workerFilePath(), { featureUid.toString() } );
+	}
 
 	m_workersMutex.lock();
 	m_workers[featureUid] = worker;
@@ -224,15 +234,6 @@ bool FeatureWorkerManager::isWorkerRunning( Feature::Uid featureUid )
 
 
 
-FeatureUidList FeatureWorkerManager::runningWorkers()
-{
-	QMutexLocker locker( &m_workersMutex );
-
-	return m_workers.keys();
-}
-
-
-
 void FeatureWorkerManager::acceptConnection()
 {
 	vDebug() << "accepting connection";
@@ -269,7 +270,7 @@ void FeatureWorkerManager::processConnection( QTcpSocket* socket )
 
 		if( message.command() >= 0 )
 		{
-			m_featureManager.handleFeatureMessage( m_server, MessageContext( socket ), message );
+			VeyonCore::featureManager().handleFeatureMessage( m_server, MessageContext( socket ), message );
 		}
 	}
 	else
