@@ -1,7 +1,7 @@
 /*
  *  LockWidget.cpp - widget for locking a client
  *
- *  Copyright (c) 2006-2021 Tobias Junghans <tobydox@veyon.io>
+ *  Copyright (c) 2006-2022 Tobias Junghans <tobydox@veyon.io>
  *
  *  This file is part of Veyon - https://veyon.io
  *
@@ -26,7 +26,6 @@
 #include "PlatformInputDeviceFunctions.h"
 
 #include <QApplication>
-#include <QDesktopWidget>
 #include <QPainter>
 #include <QScreen>
 #include <QWindow>
@@ -58,7 +57,11 @@ LockWidget::LockWidget( Mode mode, const QPixmap& background, QWidget* parent )
 	setWindowTitle( {} );
 	show();
 	move( 0, 0 );
-	setFixedSize( qApp->desktop()->size() );
+	setFixedSize(
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+		windowHandle()->
+#endif
+				  screen()->virtualSize());
 	VeyonCore::platform().coreFunctions().raiseWindow( this, true );
 	showFullScreen();
 	setFocusPolicy( Qt::StrongFocus );
