@@ -1,7 +1,7 @@
 /*
  * PluginManager.cpp - implementation of the PluginManager class
  *
- * Copyright (c) 2017-2021 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2017-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -22,6 +22,8 @@
  *
  */
 
+#include <veyonconfig.h>
+
 #include <QCoreApplication>
 #include <QDir>
 #include <QPluginLoader>
@@ -124,8 +126,6 @@ PluginUidList PluginManager::pluginUids() const
 		pluginUidList += pluginInterface->uid();
 	}
 
-	std::sort( pluginUidList.begin(), pluginUidList.end() );
-
 	return pluginUidList;
 }
 
@@ -171,16 +171,32 @@ void PluginManager::initPluginSearchPath()
 		{
 			vDebug() << "Adding plugin search path" << pluginSearchPath;
 		}
-		QDir::addSearchPath( QStringLiteral( "plugins" ), pluginSearchPath );
+		m_pluginSearchPaths.append(pluginSearchPath);
 		QCoreApplication::addLibraryPath( pluginSearchPath );
 	}
+#if defined(VEYON_WITH_TESTS)
+	for (const auto& baseDir : {QStringLiteral(CMAKE_BINARY_DIR "/plugins/platform"),
+								 QStringLiteral(CMAKE_BINARY_DIR "/plugins/vncserver"),
+								 QStringLiteral(CMAKE_BINARY_DIR "/plugins/")})
+	{
+		for (auto pluginDir : QDir(baseDir).entryInfoList(QDir::AllDirs | QDir::NoDotAndDotDot))
+		{
+			m_pluginSearchPaths.append(pluginDir.absoluteFilePath());
+		}
+	}
+#endif
 }
 
 
 
 void PluginManager::loadPlugins( const QString& nameFilter )
 {
-	const auto plugins = QDir( QStringLiteral( "plugins:" ) ).entryInfoList( { nameFilter } );
+	QFileInfoList plugins;
+	for (const auto& pluginSearchPath : m_pluginSearchPaths)
+	{
+		plugins.append(QDir(pluginSearchPath).entryInfoList({nameFilter}));
+	}
+
 	for( const auto& fileInfo : plugins )
 	{
 		const auto fileName = fileInfo.fileName();
