@@ -1,7 +1,7 @@
 /*
  * VeyonConnection.cpp - implementation of VeyonConnection
  *
- * Copyright (c) 2008-2021 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2008-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -43,16 +43,15 @@ rfbBool handleVeyonMessage( rfbClient* client, rfbServerToClientMsg* msg )
 	auto connection = reinterpret_cast<VeyonConnection *>( VncConnection::clientData( client, VeyonConnection::VeyonConnectionTag ) );
 	if( connection )
 	{
-		return connection->handleServerMessage( client, msg->type );
+		return connection->handleServerMessage( client, msg->type ) ? TRUE : FALSE;
 	}
 
-	return false;
+	return FALSE;
 }
 
 
 
-VeyonConnection::VeyonConnection( VncConnection* vncConnection ):
-	m_vncConnection( vncConnection ),
+VeyonConnection::VeyonConnection():
 	m_veyonAuthType( RfbVeyonAuth::Logon ),
 	m_user(),
 	m_userHomeDir()
@@ -75,29 +74,39 @@ VeyonConnection::VeyonConnection( VncConnection* vncConnection ):
 	}
 
 	connect( m_vncConnection, &VncConnection::connectionPrepared, this, &VeyonConnection::registerConnection, Qt::DirectConnection );
-	connect( m_vncConnection, &VncConnection::destroyed, this, &VeyonConnection::deleteLater );
+	connect( m_vncConnection, &VncConnection::destroyed, VeyonCore::instance(), [this]() {
+		delete this;
+	} );
 }
 
 
 
 VeyonConnection::~VeyonConnection()
 {
-	unregisterConnection();
-
 	delete m_authenticationProxy;
 }
 
 
 
-void VeyonConnection::sendFeatureMessage( const FeatureMessage& featureMessage, bool wake )
+void VeyonConnection::stopAndDeleteLater()
 {
-	if( m_vncConnection.isNull() )
-	{
-		vCritical() << "cannot enqueue event as VNC connection is invalid";
-		return;
-	}
+	unregisterConnection();
 
-	m_vncConnection->enqueueEvent( new VncFeatureMessageEvent( featureMessage ), wake );
+	if( m_vncConnection )
+	{
+		m_vncConnection->stopAndDeleteLater();
+		m_vncConnection = nullptr;
+	}
+}
+
+
+
+void VeyonConnection::sendFeatureMessage(const FeatureMessage& featureMessage)
+{
+	if( m_vncConnection )
+	{
+		m_vncConnection->enqueueEvent(new VncFeatureMessageEvent(featureMessage));
+	}
 }
 
 
@@ -115,8 +124,8 @@ bool VeyonConnection::handleServerMessage( rfbClient* client, uint8_t msg )
 			return false;
 		}
 
-		vDebug() << "received feature message" << featureMessage.command()
-			   << "with arguments" << featureMessage.arguments();
+		vDebug() << qUtf8Printable(QStringLiteral("%1:%2").arg(QString::fromUtf8(client->serverHost)).arg(client->serverPort))
+				 << featureMessage;
 
 		Q_EMIT featureMessageReceived( featureMessage );
 
@@ -135,7 +144,7 @@ bool VeyonConnection::handleServerMessage( rfbClient* client, uint8_t msg )
 
 void VeyonConnection::registerConnection()
 {
-	if( m_vncConnection.isNull() == false )
+	if( m_vncConnection )
 	{
 		m_vncConnection->setClientData( VeyonConnectionTag, this );
 	}
@@ -145,7 +154,7 @@ void VeyonConnection::registerConnection()
 
 void VeyonConnection::unregisterConnection()
 {
-	if( m_vncConnection.isNull() == false )
+	if( m_vncConnection )
 	{
 		m_vncConnection->setClientData( VeyonConnectionTag, nullptr );
 	}
@@ -153,11 +162,11 @@ void VeyonConnection::unregisterConnection()
 
 
 
-int8_t VeyonConnection::handleSecTypeVeyon( rfbClient* client, uint32_t authScheme )
+rfbBool VeyonConnection::handleSecTypeVeyon( rfbClient* client, uint32_t authScheme )
 {
 	if( authScheme != rfbSecTypeVeyon )
 	{
-		return false;
+		return FALSE;
 	}
 
 	hookPrepareAuthentication( client );
@@ -165,25 +174,53 @@ int8_t VeyonConnection::handleSecTypeVeyon( rfbClient* client, uint32_t authSche
 	auto connection = static_cast<VeyonConnection *>( VncConnection::clientData( client, VeyonConnectionTag ) );
 	if( connection == nullptr )
 	{
-		return false;
+		return FALSE;
 	}
 
+	const auto proxy = connection->m_authenticationProxy;
+
 	SocketDevice socketDevice( VncConnection::libvncClientDispatcher, client );
 	VariantArrayMessage message( &socketDevice );
-	message.receive();
+	if( message.receive() == false )
+	{
+		vDebug() << QThread::currentThreadId() << "invalid authentication message received";
+		if( proxy )
+		{
+			proxy->notifyProtocolError();
+		}
+		return FALSE;
+	}
 
-	int authTypeCount = message.read().toInt();
+	const auto authTypeCount = message.read().toInt();
+
+	if( authTypeCount == 0 )
+	{
+		vDebug() << QThread::currentThreadId() << "no auth types received";
+		if( proxy )
+		{
+			proxy->notifyProtocolError();
+		}
+		return FALSE;
+	}
 
 	QList<RfbVeyonAuth::Type> authTypes;
 	authTypes.reserve( authTypeCount );
 
 	for( int i = 0; i < authTypeCount; ++i )
 	{
-		authTypes.append( QVariantHelper<RfbVeyonAuth::Type>::value( message.read() ) );
+		const auto authType = QVariantHelper<RfbVeyonAuth::Type>::value( message.read() );
+		if( authType == RfbVeyonAuth::Type::Invalid )
+		{
+			vDebug() << QThread::currentThreadId() << "invalid auth type received";
+			if( proxy )
+			{
+				proxy->notifyProtocolError();
+			}
+			return FALSE;
+		}
+		authTypes.append( authType );
 	}
 
-	auto proxy = connection->m_authenticationProxy;
-
 	if( proxy )
 	{
 		proxy->setAuthenticationTypes( authTypes );
@@ -191,32 +228,21 @@ int8_t VeyonConnection::handleSecTypeVeyon( rfbClient* client, uint32_t authSche
 
 	vDebug() << QThread::currentThreadId() << "received authentication types:" << authTypes;
 
-	RfbVeyonAuth::Type chosenAuthType = RfbVeyonAuth::Token;
-	if( authTypes.count() > 0 )
-	{
-		chosenAuthType = authTypes.first();
-
-		// look whether the VncConnection recommends a specific
-		// authentication type (e.g. VeyonAuthHostBased when running as
-		// demo client)
-
-		for( auto authType : authTypes )
-		{
-			if( connection->veyonAuthType() == authType )
-			{
-				chosenAuthType = authType;
-			}
-		}
-	}
-
+	auto chosenAuthType = authTypes.first();
 	if( proxy )
 	{
 		chosenAuthType = proxy->initCredentials();
 	}
-
-	if( chosenAuthType == RfbVeyonAuth::None )
+	// look whether the VncConnection recommends a specific authentication type (e.g. RfbVeyonAuth::Token
+	// when running as demo client)
+	else if( authTypes.contains( connection->veyonAuthType() ) )
 	{
-		return false;
+		chosenAuthType = connection->veyonAuthType();
+	}
+
+	if( chosenAuthType == RfbVeyonAuth::Invalid )
+	{
+		return FALSE;
 	}
 
 	vDebug() << QThread::currentThreadId() << "chose authentication type:" << authTypes;
@@ -238,7 +264,11 @@ int8_t VeyonConnection::handleSecTypeVeyon( rfbClient* client, uint32_t authSche
 	authReplyMessage.send();
 
 	VariantArrayMessage authAckMessage( &socketDevice );
-	authAckMessage.receive();
+	if( authAckMessage.receive() == false )
+	{
+		vWarning() << QThread::currentThreadId() << "failed to receive authentication acknowledge";
+		return FALSE;
+	}
 
 	switch( chosenAuthType )
 	{
@@ -252,7 +282,7 @@ int8_t VeyonConnection::handleSecTypeVeyon( rfbClient* client, uint32_t authSche
 			if( challenge.size() != CryptoCore::ChallengeSize )
 			{
 				vCritical() << QThread::currentThreadId() << "challenge size mismatch!";
-				return false;
+				return FALSE;
 			}
 
 			// create local copy of private key so we can modify it within our own thread
@@ -260,7 +290,7 @@ int8_t VeyonConnection::handleSecTypeVeyon( rfbClient* client, uint32_t authSche
 			if( key.isNull() || key.canSign() == false )
 			{
 				vCritical() << QThread::currentThreadId() << "invalid private key!";
-				return false;
+				return FALSE;
 			}
 
 			const auto signature = key.signMessage( challenge, CryptoCore::DefaultSignatureAlgorithm );
@@ -282,7 +312,7 @@ int8_t VeyonConnection::handleSecTypeVeyon( rfbClient* client, uint32_t authSche
 		if( publicKey.canEncrypt() == false )
 		{
 			vCritical() << QThread::currentThreadId() << "can't encrypt with given public key!";
-			return false;
+			return FALSE;
 		}
 
 		CryptoCore::SecureArray plainTextPassword( connection->authenticationCredentials().logonPassword() );
@@ -290,7 +320,7 @@ int8_t VeyonConnection::handleSecTypeVeyon( rfbClient* client, uint32_t authSche
 		if( encryptedPassword.isEmpty() )
 		{
 			vCritical() << QThread::currentThreadId() << "password encryption failed!";
-			return false;
+			return FALSE;
 		}
 
 		VariantArrayMessage passwordResponse( &socketDevice );
@@ -312,7 +342,7 @@ int8_t VeyonConnection::handleSecTypeVeyon( rfbClient* client, uint32_t authSche
 		break;
 	}
 
-	return true;
+	return TRUE;
 }
 
 
