@@ -1,7 +1,7 @@
 /*
  * VeyonCore.cpp - implementation of Veyon Core
  *
- * Copyright (c) 2006-2021 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2006-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -29,12 +29,16 @@
 #include <QApplication>
 #include <QDir>
 #include <QGroupBox>
+#include <QJsonDocument>
 #include <QLabel>
+#include <QLibraryInfo>
 #include <QProcessEnvironment>
+#include <QRegularExpression>
 #include <QSysInfo>
 
 #include "BuiltinFeatures.h"
 #include "ComputerControlInterface.h"
+#include "FeatureManager.h"
 #include "Filesystem.h"
 #include "HostAddress.h"
 #include "Logger.h"
@@ -74,8 +78,6 @@ VeyonCore::VeyonCore( QCoreApplication* application, Component component, const
 
 	s_instance = this;
 
-	setupApplicationParameters();
-
 	initPlatformPlugin();
 
 	initConfiguration();
@@ -103,6 +105,9 @@ VeyonCore::VeyonCore( QCoreApplication* application, Component component, const
 
 VeyonCore::~VeyonCore()
 {
+	delete m_featureManager;
+	m_featureManager = nullptr;
+
 	delete m_userGroupsBackendManager;
 	m_userGroupsBackendManager = nullptr;
 
@@ -112,15 +117,15 @@ VeyonCore::~VeyonCore()
 	delete m_builtinFeatures;
 	m_builtinFeatures = nullptr;
 
+	delete m_logger;
+	m_logger = nullptr;
+
 	delete m_platformPluginManager;
 	m_platformPluginManager = nullptr;
 
 	delete m_pluginManager;
 	m_pluginManager = nullptr;
 
-	delete m_logger;
-	m_logger = nullptr;
-
 	delete m_config;
 	m_config = nullptr;
 
@@ -137,14 +142,12 @@ VeyonCore::~VeyonCore()
 
 VeyonCore* VeyonCore::instance()
 {
-	Q_ASSERT(s_instance != nullptr);
-
 	return s_instance;
 }
 
 
 
-QString VeyonCore::version()
+QString VeyonCore::versionString()
 {
 	return QStringLiteral( VEYON_VERSION );
 }
@@ -167,11 +170,13 @@ QString VeyonCore::translationsDirectory()
 
 QString VeyonCore::qtTranslationsDirectory()
 {
-#ifdef QT_TRANSLATIONS_DIR
-	return QStringLiteral( QT_TRANSLATIONS_DIR );
-#else
+	const auto path = QLibraryInfo::location(QLibraryInfo::TranslationsPath);
+	if( QDir{path}.exists() )
+	{
+		return path;
+	}
+
 	return translationsDirectory();
-#endif
 }
 
 
@@ -203,7 +208,11 @@ void VeyonCore::setupApplicationParameters()
 	QCoreApplication::setOrganizationDomain( QStringLiteral( "veyon.io" ) );
 	QCoreApplication::setApplicationName( QStringLiteral( "Veyon" ) );
 
+	QCoreApplication::setAttribute( Qt::AA_ShareOpenGLContexts );
+
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 	QApplication::setAttribute( Qt::AA_UseHighDpiPixmaps );
+#endif
 }
 
 
@@ -232,31 +241,31 @@ void VeyonCore::enforceBranding( QWidget *topLevelWidget )
 {
 	const auto appName = QStringLiteral( "Veyon" );
 
-	auto labels = topLevelWidget->findChildren<QLabel *>();
+	const auto labels = topLevelWidget->findChildren<QLabel *>();
 	for( auto label : labels )
 	{
 		label->setText( label->text().replace( appName, VeyonCore::applicationName() ) );
 	}
 
-	auto buttons = topLevelWidget->findChildren<QAbstractButton *>();
+	const auto buttons = topLevelWidget->findChildren<QAbstractButton *>();
 	for( auto button : buttons )
 	{
 		button->setText( button->text().replace( appName, VeyonCore::applicationName() ) );
 	}
 
-	auto groupBoxes = topLevelWidget->findChildren<QGroupBox *>();
+	const auto groupBoxes = topLevelWidget->findChildren<QGroupBox *>();
 	for( auto groupBox : groupBoxes )
 	{
 		groupBox->setTitle( groupBox->title().replace( appName, VeyonCore::applicationName() ) );
 	}
 
-	auto actions = topLevelWidget->findChildren<QAction *>();
+	const auto actions = topLevelWidget->findChildren<QAction *>();
 	for( auto action : actions )
 	{
 		action->setText( action->text().replace( appName, VeyonCore::applicationName() ) );
 	}
 
-	auto widgets = topLevelWidget->findChildren<QWidget *>();
+	const auto widgets = topLevelWidget->findChildren<QWidget *>();
 	for( auto widget : widgets )
 	{
 		widget->setWindowTitle( widget->windowTitle().replace( appName, VeyonCore::applicationName() ) );
@@ -269,12 +278,12 @@ void VeyonCore::enforceBranding( QWidget *topLevelWidget )
 
 bool VeyonCore::isDebugging()
 {
-	return instance()->m_debugging;
+	return s_instance && s_instance->m_debugging;
 }
 
 
 
-QByteArray VeyonCore::shortenFuncinfo( QByteArray info )
+QByteArray VeyonCore::shortenFuncinfo( const QByteArray& info )
 {
 	const auto funcinfo = cleanupFuncinfo( info );
 
@@ -470,9 +479,31 @@ QString VeyonCore::formattedUuid( QUuid uuid )
 
 
 
+QString VeyonCore::stringify( const QVariantMap& map )
+{
+	return QString::fromUtf8( QJsonDocument(QJsonObject::fromVariantMap(map)).toJson(QJsonDocument::Compact) );
+}
+
+
+
+QString VeyonCore::screenName(const QScreen& screen, int index)
+{
+	auto screenName = tr("Screen %1").arg(index);
+
+	const auto displayDeviceName = platform().coreFunctions().queryDisplayDeviceName(screen);
+	if(displayDeviceName.isEmpty() == false)
+	{
+		screenName.append(QStringLiteral(" – %1").arg(displayDeviceName));
+	}
+
+	return screenName;
+}
+
+
+
 bool VeyonCore::isAuthenticationKeyNameValid( const QString& authKeyName )
 {
-	return QRegExp( QStringLiteral("\\w+") ).exactMatch( authKeyName );
+	return QRegularExpression( QStringLiteral("^[\\w-]+$") ).match( authKeyName ).hasMatch();
 }
 
 
@@ -507,7 +538,7 @@ void VeyonCore::initPlatformPlugin()
 
 void VeyonCore::initSession()
 {
-	if( config().multiSessionModeEnabled() )
+	if( component() != Component::Service && config().multiSessionModeEnabled() )
 	{
 		const auto systemEnv = QProcessEnvironment::systemEnvironment();
 		if( systemEnv.contains( sessionIdEnvironmentVariable() ) )
@@ -570,18 +601,17 @@ void VeyonCore::initLogging( const QString& appComponentName )
 
 void VeyonCore::initLocaleAndTranslation()
 {
-	TranslationLoader::load( QStringLiteral("qt") );
-
-	const auto configuredLocale = TranslationLoader::load( QStringLiteral("veyon") );
-
-	if( configuredLocale.language() == QLocale::Hebrew ||
-		configuredLocale.language() == QLocale::Arabic )
+	if( TranslationLoader::load( QStringLiteral("qtbase") ) == false )
 	{
-		auto app = qobject_cast<QApplication *>( QCoreApplication::instance() );
-		if( app )
-		{
-			QApplication::setLayoutDirection( Qt::RightToLeft );
-		}
+		TranslationLoader::load( QStringLiteral("qt") );
+	}
+
+	TranslationLoader::load( QStringLiteral("veyon") );
+
+	const auto app = qobject_cast<QGuiApplication *>( QCoreApplication::instance() );
+	if( app )
+	{
+		QGuiApplication::setLayoutDirection( QLocale{}.textDirection() );
 	}
 }
 
@@ -620,6 +650,7 @@ void VeyonCore::initPlugins()
 
 void VeyonCore::initManagers()
 {
+	m_featureManager = new FeatureManager(this);
 	m_userGroupsBackendManager = new UserGroupsBackendManager( this );
 	m_networkObjectDirectoryManager = new NetworkObjectDirectoryManager( this );
 }
@@ -656,6 +687,7 @@ bool VeyonCore::initKeyFileAuthentication()
 				m_authenticationCredentials->loadPrivateKey( VeyonCore::filesystem().privateKeyPath( authKeyName ) ) )
 		{
 			m_authenticationCredentials->setAuthenticationKeyName( authKeyName );
+			return true;
 		}
 	}
 	else
@@ -681,7 +713,7 @@ bool VeyonCore::initKeyFileAuthentication()
 
 void VeyonCore::initSystemInfo()
 {
-	vDebug() << version() << HostAddress::localFQDN()
+	vDebug() << versionString() << HostAddress::localFQDN()
 			 << QSysInfo::kernelType() << QSysInfo::kernelVersion()
 			 << QSysInfo::prettyProductName() << QSysInfo::productType() << QSysInfo::productVersion();
 }
