@@ -1,7 +1,7 @@
 /*
  * VncClientProtocol.cpp - implementation of the VncClientProtocol class
  *
- * Copyright (c) 2017-2021 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2017-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -30,6 +30,7 @@ extern "C"
 
 #include <QBuffer>
 #include <QRegion>
+#include <QRegularExpression>
 #include <QTcpSocket>
 
 #include "VncClientProtocol.h"
@@ -66,7 +67,7 @@ vncEncryptBytes(unsigned char *bytes, const char *passwd, size_t passwd_length)
 
 
 
-VncClientProtocol::VncClientProtocol( QTcpSocket* socket, const Password& vncPassword ) :
+VncClientProtocol::VncClientProtocol( QIODevice* socket, const Password& vncPassword ) :
 	m_socket( socket ),
 	m_state( Disconnected ),
 	m_vncPassword( vncPassword ),
@@ -240,11 +241,12 @@ bool VncClientProtocol::readProtocol()
 			return false;
 		}
 
-		QRegExp protocolRX( QStringLiteral("RFB (\\d\\d\\d)\\.(\\d\\d\\d)\n") );
+		const auto match = QRegularExpression{ QStringLiteral("RFB (\\d\\d\\d)\\.(\\d\\d\\d)\n") }
+							   .match( QString::fromUtf8( protocol ) );
 
-		if( protocolRX.indexIn( QString::fromUtf8( protocol ) ) != 0 ||
-			protocolRX.cap( 1 ).toInt() != 3 ||
-			protocolRX.cap( 2 ).toInt() < 7 )
+		if( match.hasMatch() == false ||
+			match.captured( 1 ).toInt() != 3 ||
+			match.captured( 2 ).toInt() < 7 )
 		{
 			vCritical() << "invalid protocol version";
 			m_socket->close();
@@ -564,12 +566,12 @@ bool VncClientProtocol::handleRect( QBuffer& buffer, rfbFramebufferUpdateRectHea
 	case rfbEncodingXCursor:
 		return width * height == 0 ||
 				( buffer.read( sz_rfbXCursorColors ).size() == sz_rfbXCursorColors &&
-				  buffer.read( 2 * bytesPerRow * height ).size() == static_cast<int>( 2 * bytesPerRow * height ) );
+				  buffer.read( size_t(2) * bytesPerRow * height ).size() == static_cast<int>( 2 * bytesPerRow * height ) );
 
 	case rfbEncodingRichCursor:
 		return width * height == 0 ||
-				( buffer.read( width * height * bytesPerPixel ).size() == static_cast<int>( width * height * bytesPerPixel ) &&
-				  buffer.read( bytesPerRow * height ).size() == static_cast<int>( bytesPerRow * height ) );
+				( buffer.read( size_t(width) * height * bytesPerPixel ).size() == static_cast<int>( width * height * bytesPerPixel ) &&
+				  buffer.read( size_t(bytesPerRow) * height ).size() == static_cast<int>( bytesPerRow * height ) );
 
 	case rfbEncodingSupportedMessages:
 		return buffer.read( sz_rfbSupportedMessages ).size() == sz_rfbSupportedMessages;
@@ -580,7 +582,7 @@ bool VncClientProtocol::handleRect( QBuffer& buffer, rfbFramebufferUpdateRectHea
 		return buffer.read( width ).size() == static_cast<int>( width );
 
 	case rfbEncodingRaw:
-		return buffer.read( width * height * bytesPerPixel ).size() == static_cast<int>( width * height * bytesPerPixel );
+		return buffer.read( size_t(width) * height * bytesPerPixel ).size() == static_cast<int>( width * height * bytesPerPixel );
 
 	case rfbEncodingCopyRect:
 		return buffer.read( sz_rfbCopyRect ).size() == sz_rfbCopyRect;
@@ -632,7 +634,7 @@ bool VncClientProtocol::handleRectEncodingRRE( QBuffer& buffer, uint bytesPerPix
 	const auto rectDataSize = qFromBigEndian( hdr.nSubrects ) * ( bytesPerPixel + sz_rfbRectangle );
 	const auto totalDataSize = static_cast<int>( bytesPerPixel + rectDataSize );
 
-	return buffer.read( totalDataSize ).size() == totalDataSize;
+	return totalDataSize < MaxMessageSize && buffer.read( totalDataSize ).size() == totalDataSize;
 }
 
 
@@ -649,7 +651,7 @@ bool VncClientProtocol::handleRectEncodingCoRRE( QBuffer& buffer, uint bytesPerP
 	const auto rectDataSize = qFromBigEndian( hdr.nSubrects ) * ( bytesPerPixel + 4 );
 	const auto totalDataSize = static_cast<int>( bytesPerPixel + rectDataSize );
 
-	return buffer.read( totalDataSize ).size() == totalDataSize;
+	return totalDataSize < MaxMessageSize && buffer.read( totalDataSize ).size() == totalDataSize;
 
 }
 
@@ -755,7 +757,7 @@ bool VncClientProtocol::handleRectEncodingZlib( QBuffer& buffer )
 
 	const auto n = qFromBigEndian( hdr.nBytes );
 
-	return buffer.read( n ).size() == static_cast<int>( n );
+	return n < MaxMessageSize && uint32_t(buffer.read(n).size()) == n;
 }
 
 
@@ -771,7 +773,7 @@ bool VncClientProtocol::handleRectEncodingZRLE(QBuffer &buffer)
 
 	const auto n = qFromBigEndian( hdr.length );
 
-	return buffer.read( n ).size() == static_cast<int>( n );
+	return n < MaxMessageSize && uint32_t(buffer.read(n).size()) == n;
 }
 
 
