@@ -1,7 +1,7 @@
 /*
  * VncView.cpp - abstract base for all VNC views
  *
- * Copyright (c) 2006-2021 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2006-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -32,13 +32,23 @@
 
 #include "PlatformInputDeviceFunctions.h"
 #include "KeyboardShortcutTrapper.h"
+#include "VeyonConnection.h"
 #include "VncConnection.h"
 #include "VncView.h"
 
 
-VncView::VncView( VncConnection* connection ) :
-	m_connection( connection ),
-	m_framebufferSize( connection->image().size() ),
+VncView::VncView( const ComputerControlInterface::Pointer& computerControlInterface ) :
+	m_computerControlInterface( [computerControlInterface]() {
+		if( computerControlInterface->state() == ComputerControlInterface::State::Disconnected ||
+			computerControlInterface->connection() == nullptr )
+		{
+			computerControlInterface->start();
+		}
+		return computerControlInterface;
+	}() ),
+	m_previousUpdateMode( m_computerControlInterface->updateMode() ),
+	m_connection( computerControlInterface->connection()->vncConnection() ),
+	m_framebufferSize( m_connection->image().size() ),
 	m_keyboardShortcutTrapper( VeyonCore::platform().inputDeviceFunctions().createKeyboardShortcutTrapper( nullptr ) )
 {
 	// handle/forward trapped keyboard shortcuts
@@ -46,12 +56,18 @@ VncView::VncView( VncConnection* connection ) :
 					  m_keyboardShortcutTrapper, [this]( KeyboardShortcutTrapper::Shortcut shortcut ) {
 						  handleShortcut( shortcut );
 					  } );
+
+	m_computerControlInterface->setUpdateMode( ComputerControlInterface::UpdateMode::Live );
 }
 
 
 
 VncView::~VncView()
 {
+	unpressModifiers();
+
+	m_computerControlInterface->setUpdateMode( m_previousUpdateMode );
+
 	delete m_keyboardShortcutTrapper;
 }
 
@@ -83,24 +99,42 @@ QSize VncView::effectiveFramebufferSize() const
 
 
 
+void VncView::setViewport(QRect viewport)
+{
+	if( m_viewport != viewport )
+	{
+		m_viewport = viewport;
+
+		updateGeometry();
+	}
+}
+
+
+
 void VncView::setViewOnly( bool viewOnly )
 {
 	if( viewOnly == m_viewOnly )
 	{
 		return;
 	}
+
 	m_viewOnly = viewOnly;
 
+	if( m_connection )
+	{
+		m_connection->setUseRemoteCursor( !viewOnly );
+	}
+
 	if( m_viewOnly )
 	{
 		m_keyboardShortcutTrapper->setEnabled( false );
-		updateLocalCursor();
 	}
 	else
 	{
-		updateLocalCursor();
 		m_keyboardShortcutTrapper->setEnabled( true );
 	}
+
+	updateLocalCursor();
 }
 
 
@@ -224,24 +258,6 @@ QRect VncView::mapFromFramebuffer( QRect r )
 
 
 
-void VncView::updateCursorPos( int x, int y )
-{
-	if( viewOnly() )
-	{
-		if( m_cursorShape.isNull() == false )
-		{
-			updatePaintedCursor();
-		}
-		m_cursorPos = { x, y };
-		if( m_cursorShape.isNull() == false )
-		{
-			updatePaintedCursor();
-		}
-	}
-}
-
-
-
 void VncView::updateCursorShape( const QPixmap& cursorShape, int xh, int yh )
 {
 	const auto scale = scaleFactor();
@@ -251,11 +267,6 @@ void VncView::updateCursorShape( const QPixmap& cursorShape, int xh, int yh )
 										int( cursorShape.height()*scale ),
 										Qt::IgnoreAspectRatio, Qt::SmoothTransformation );
 
-	if( viewOnly() )
-	{
-		updateView( m_cursorPos.x(), m_cursorPos.y(), m_cursorShape.width(), m_cursorShape.height() );
-	}
-
 	updateLocalCursor();
 }
 
@@ -264,6 +275,8 @@ void VncView::updateCursorShape( const QPixmap& cursorShape, int xh, int yh )
 void VncView::updateFramebufferSize( int w, int h )
 {
 	m_framebufferSize = QSize( w, h );
+
+	updateGeometry();
 }
 
 
@@ -566,7 +579,7 @@ void VncView::mouseEventHandler( QMouseEvent* event )
 
 	static constexpr std::array<ButtonTranslation, 3> buttonTranslationMap{ {
 		{ Qt::LeftButton, rfbButton1Mask },
-		{ Qt::MidButton, rfbButton2Mask },
+		{ Qt::MiddleButton, rfbButton2Mask },
 		{ Qt::RightButton, rfbButton3Mask }
 	} };
 
@@ -629,13 +642,6 @@ void VncView::updateLocalCursor()
 
 
 
-void VncView::updatePaintedCursor()
-{
-	updateView( m_cursorPos.x(), m_cursorPos.y(), m_cursorShape.width(), m_cursorShape.height() );
-}
-
-
-
 void VncView::pressKey( unsigned int key )
 {
 	m_connection->keyEvent( key, true );
