@@ -1,7 +1,7 @@
 /*
  * AuthKeysManager.cpp - implementation of AuthKeysManager class
  *
- * Copyright (c) 2018-2021 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2018-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -24,6 +24,7 @@
 
 #include <QDir>
 #include <QFileInfo>
+#include <QRegularExpression>
 
 #include "AuthKeysManager.h"
 #include "CommandLineIO.h"
@@ -460,11 +461,12 @@ QString AuthKeysManager::exportedKeyFileName( const QString& name, const QString
 
 QString AuthKeysManager::keyNameFromExportedKeyFile( const QString& keyFile )
 {
-	QRegExp rx( QStringLiteral("^(.*)_(.*)_key.pem$") );
+	const auto keyNameMatch = QRegularExpression( QStringLiteral("^(.*)_(.*)_key.pem$") )
+								  .match(QFileInfo(keyFile).fileName());
 
-	if( rx.indexIn( QFileInfo( keyFile ).fileName() ) == 0 )
+	if( keyNameMatch.hasMatch() )
 	{
-		return rx.cap( 1 );
+		return keyNameMatch.captured( 1 );
 	}
 
 	return {};
