@@ -1,7 +1,7 @@
 /*
  * DemoClient.cpp - client widget for demo mode
  *
- * Copyright (c) 2006-2021 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2006-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -23,9 +23,7 @@
  */
 
 #include <QApplication>
-#include <QDesktopWidget>
 #include <QIcon>
-#include <QLayout>
 
 #include "DemoClient.h"
 #include "VeyonConfiguration.h"
@@ -34,9 +32,9 @@
 #include "VncViewWidget.h"
 
 
-DemoClient::DemoClient( const QString& host, int port, bool fullscreen, const QRect& viewport, QObject* parent ) :
+DemoClient::DemoClient( const QString& host, int port, bool fullscreen, QRect viewport, QObject* parent ) :
 	QObject( parent ),
-	m_toplevel( nullptr )
+	m_computerControlInterface( ComputerControlInterface::Pointer::create( Computer( {}, host, host ), port, this ) )
 {
 	if( fullscreen )
 	{
@@ -50,21 +48,14 @@ DemoClient::DemoClient( const QString& host, int port, bool fullscreen, const QR
 	m_toplevel->setWindowTitle( tr( "%1 Demo" ).arg( VeyonCore::applicationName() ) );
 	m_toplevel->setWindowIcon( QPixmap( QStringLiteral(":/core/icon64.png") ) );
 	m_toplevel->setAttribute( Qt::WA_DeleteOnClose, false );
+	m_toplevel->installEventFilter(this);
 
 	if( fullscreen == false )
 	{
 		m_toplevel->setWindowFlags( Qt::Window | Qt::CustomizeWindowHint | Qt::WindowTitleHint | Qt::WindowMinMaxButtonsHint );
-		m_toplevel->resize( QApplication::desktop()->availableGeometry( m_toplevel ).size() - QSize( 10, 30 ) );
 	}
 
-	m_vncView = new VncViewWidget( host, port, m_toplevel, VncView::DemoMode, viewport );
-
-	auto toplevelLayout = new QVBoxLayout;
-	toplevelLayout->setContentsMargins( 0, 0, 0, 0 );
-	toplevelLayout->setSpacing( 0 );
-	toplevelLayout->addWidget( m_vncView );
-
-	m_toplevel->setLayout( toplevelLayout );
+	m_vncView = new VncViewWidget( m_computerControlInterface, viewport, m_toplevel );
 
 	connect( m_toplevel, &QObject::destroyed, this, &DemoClient::viewDestroyed );
 	connect( m_vncView, &VncViewWidget::sizeHintChanged, this, &DemoClient::resizeToplevelWidget );
@@ -79,6 +70,8 @@ DemoClient::DemoClient( const QString& host, int port, bool fullscreen, const QR
 		m_toplevel->show();
 	}
 
+	resizeToplevelWidget();
+
 	VeyonCore::platform().coreFunctions().raiseWindow( m_toplevel, fullscreen );
 
 	VeyonCore::platform().coreFunctions().disableScreenSaver();
@@ -95,6 +88,19 @@ DemoClient::~DemoClient()
 
 
 
+bool DemoClient::eventFilter(QObject* watched, QEvent* event)
+{
+	if (watched == m_toplevel && event->type() == QEvent::Resize)
+	{
+		m_vncView->setFixedSize(m_toplevel->size());
+		return true;
+	}
+
+	return QObject::eventFilter(watched, event);
+}
+
+
+
 void DemoClient::viewDestroyed( QObject* obj )
 {
 	// prevent double deletion of toplevel widget
@@ -112,10 +118,10 @@ void DemoClient::resizeToplevelWidget()
 {
 	if( m_toplevel->windowState() & Qt::WindowFullScreen )
 	{
-		m_vncView->resize( m_toplevel->size() );
+		m_vncView->setFixedSize(m_toplevel->size());
 	}
 	else
 	{
-		m_toplevel->resize( m_vncView->sizeHint() );
+		m_toplevel->resize(m_vncView->sizeHint());
 	}
 }
