@@ -1,7 +1,7 @@
 /*
  * DemoFeaturePlugin.cpp - implementation of DemoFeaturePlugin class
  *
- * Copyright (c) 2017-2021 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2017-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -35,7 +35,7 @@
 #include "FeatureWorkerManager.h"
 #include "HostAddress.h"
 #include "Logger.h"
-#include "FeatureWorkerManager.h"
+#include "PlatformSessionFunctions.h"
 #include "VeyonConfiguration.h"
 #include "VeyonMasterInterface.h"
 #include "VeyonServerInterface.h"
@@ -44,23 +44,23 @@
 DemoFeaturePlugin::DemoFeaturePlugin( QObject* parent ) :
 	QObject( parent ),
 	m_demoFeature( QStringLiteral( "Demo" ),
-				   Feature::Mode | Feature::AllComponents,
+				   Feature::Flag::Mode | Feature::Flag::AllComponents,
 				   Feature::Uid( "6f4cd922-b63e-40bf-9451-536065c7cdf9" ),
 				   Feature::Uid(),
 				   tr( "Demo" ), tr( "Stop demo" ),
 				   tr( "Share your screen or allow a user to share his screen with other users." ),
 				   QStringLiteral(":/demo/demo.png") ),
 	m_demoClientFullScreenFeature( QStringLiteral( "FullScreenDemo" ),
-								   Feature::Internal | Feature::AllComponents,
+								   Feature::Flag::Meta | Feature::Flag::AllComponents,
 								   Feature::Uid( "7b6231bd-eb89-45d3-af32-f70663b2f878" ),
 								   {}, tr("Full screen demo"), {}, {} ),
 	m_demoClientWindowFeature( QStringLiteral( "WindowDemo" ),
-							   Feature::Internal | Feature::AllComponents,
+							   Feature::Flag::Meta | Feature::Flag::AllComponents,
 							   Feature::Uid( "ae45c3db-dc2e-4204-ae8b-374cdab8c62c" ),
 							   {}, tr("Window demo"), {}, {} ),
 	m_shareOwnScreenFullScreenFeature( QStringLiteral( "ShareOwnScreenFullScreen" ),
-									   Feature::Mode | Feature::AllComponents,
-									   Feature::Uid( "7b6231bd-eb89-45d3-af32-f70663b2f878" ),
+									   Feature::Flag::Mode | Feature::Flag::AllComponents,
+									   Feature::Uid( "07b375e1-8ab6-4b48-bcb7-75fb3d56035b" ),
 									   m_demoFeature.uid(),
 									   tr( "Share your own screen in fullscreen mode" ), {},
 									   tr( "In this mode your screen is being displayed in "
@@ -68,8 +68,8 @@ DemoFeaturePlugin::DemoFeaturePlugin( QObject* parent ) :
 										   "devices of the users are locked." ),
 									   QStringLiteral(":/demo/presentation-fullscreen.png") ),
 	m_shareOwnScreenWindowFeature( QStringLiteral( "ShareOwnScreenWindow" ),
-								   Feature::Mode | Feature::AllComponents,
-								   Feature::Uid( "ae45c3db-dc2e-4204-ae8b-374cdab8c62c" ),
+								   Feature::Flag::Mode | Feature::Flag::AllComponents,
+								   Feature::Uid( "68c55fb9-127e-4c9f-9c90-28b998bf1a47" ),
 								   m_demoFeature.uid(),
 								   tr( "Share your own screen in a window" ), {},
 								   tr( "In this mode your screen being displayed in a "
@@ -77,7 +77,7 @@ DemoFeaturePlugin::DemoFeaturePlugin( QObject* parent ) :
 									   "able to switch to other windows as needed." ),
 								   QStringLiteral(":/demo/presentation-window.png") ),
 	m_shareUserScreenFullScreenFeature( QStringLiteral( "ShareUserScreenFullScreen" ),
-										Feature::Mode | Feature::AllComponents,
+										Feature::Flag::Mode | Feature::Flag::AllComponents,
 										Feature::Uid( "b4e542e2-1deb-48ac-910a-bbf8ac9a0bde" ),
 										m_demoFeature.uid(),
 										tr( "Share selected user's screen in fullscreen mode" ), {},
@@ -86,7 +86,7 @@ DemoFeaturePlugin::DemoFeaturePlugin( QObject* parent ) :
 											"devices of the users are locked." ),
 										QStringLiteral(":/demo/presentation-fullscreen.png") ),
 	m_shareUserScreenWindowFeature( QStringLiteral( "ShareUserScreenWindow" ),
-									Feature::Mode | Feature::AllComponents,
+									Feature::Flag::Mode | Feature::Flag::AllComponents,
 									Feature::Uid( "ebfc5ec4-f725-4bfc-a93a-c6d4864c6806" ),
 									m_demoFeature.uid(),
 									tr( "Share selected user's screen in a window" ), {},
@@ -95,7 +95,7 @@ DemoFeaturePlugin::DemoFeaturePlugin( QObject* parent ) :
 										"able to switch to other windows as needed." ),
 									QStringLiteral(":/demo/presentation-window.png") ),
 	m_demoServerFeature( QStringLiteral( "DemoServer" ),
-						 Feature::Session | Feature::Service | Feature::Worker,
+						 Feature::Flag::Session | Feature::Flag::Service | Feature::Flag::Worker,
 						 Feature::Uid( "e4b6e743-1f5b-491d-9364-e091086200f4" ),
 						 m_demoFeature.uid(),
 						 {}, {}, {} ),
@@ -106,19 +106,37 @@ DemoFeaturePlugin::DemoFeaturePlugin( QObject* parent ) :
 		m_shareUserScreenFullScreenFeature, m_shareUserScreenWindowFeature
 	} ),
 	m_demoAccessToken( CryptoCore::generateChallenge() ),
-	m_demoClientHosts(),
 	m_configuration( &VeyonCore::config() ),
 	m_demoServer( nullptr ),
 	m_demoClient( nullptr )
 {
 	connect( qGuiApp, &QGuiApplication::screenAdded, this, &DemoFeaturePlugin::addScreen );
 	connect( qGuiApp, &QGuiApplication::screenRemoved, this, &DemoFeaturePlugin::removeScreen );
+	connect( &m_demoServerControlTimer, &QTimer::timeout, this, &DemoFeaturePlugin::controlDemoServer );
 
 	updateFeatures();
 }
 
 
 
+Feature::Uid DemoFeaturePlugin::metaFeature( Feature::Uid featureUid ) const
+{
+	if( featureUid == m_shareOwnScreenFullScreenFeature.uid() ||
+		featureUid == m_shareUserScreenFullScreenFeature.uid() )
+	{
+		return m_demoClientFullScreenFeature.uid();
+	}
+	else if( featureUid == m_shareOwnScreenWindowFeature.uid() ||
+			 featureUid == m_shareUserScreenWindowFeature.uid() )
+	{
+		return m_demoClientWindowFeature.uid();
+	}
+
+	return FeatureProviderInterface::metaFeature( featureUid );
+}
+
+
+
 bool DemoFeaturePlugin::controlFeature( Feature::Uid featureUid,
 									   Operation operation,
 									   const QVariantMap& arguments,
@@ -126,7 +144,25 @@ bool DemoFeaturePlugin::controlFeature( Feature::Uid featureUid,
 {
 	if( featureUid == m_demoServerFeature.uid() )
 	{
-		return controlDemoServer( operation, arguments, computerControlInterfaces );
+		m_demoServerArguments = arguments;
+
+		if( operation == Operation::Start )
+		{
+			m_demoServerControlTimer.start( DemoServerControlInterval );
+			m_demoServerControlInterfaces = computerControlInterfaces;
+		}
+		else if( operation == Operation::Stop )
+		{
+			m_demoServerControlTimer.stop();
+		}
+		else
+		{
+			return false;
+		}
+
+		controlDemoServer();
+
+		return true;
 	}
 
 	if( featureUid == m_demoClientFullScreenFeature.uid() || featureUid == m_demoClientWindowFeature.uid() )
@@ -144,28 +180,29 @@ bool DemoFeaturePlugin::startFeature( VeyonMasterInterface& master, const Featur
 {
 	if( feature == m_shareOwnScreenWindowFeature || feature == m_shareOwnScreenFullScreenFeature )
 	{
-		// start demo server
-		controlFeature( m_demoServerFeature.uid(), Operation::Start, {},
-						{ master.localSessionControlInterface().weakPointer() } );
-
 		// start demo clients
 		controlFeature( feature == m_shareOwnScreenFullScreenFeature ? m_demoClientFullScreenFeature.uid()
 																	 : m_demoClientWindowFeature.uid(),
 						Operation::Start, {}, computerControlInterfaces );
 
+		// start demo server
+		controlFeature( m_demoServerFeature.uid(), Operation::Start, {},
+						{ master.localSessionControlInterface().weakPointer() } );
+
 		return true;
 	}
 
 	if( feature == m_shareUserScreenWindowFeature || feature == m_shareUserScreenFullScreenFeature )
 	{
-		if( master.selectedComputerControlInterfaces().size() < 1 )
+		const auto selectedComputerControlInterfaces = master.selectedComputerControlInterfaces();
+		if( selectedComputerControlInterfaces.size() < 1 )
 		{
 			QMessageBox::critical( master.mainWindow(), feature.name(),
 								   tr( "Please select a user screen to share.") );
 			return true;
 		}
 
-		if( master.selectedComputerControlInterfaces().size() > 1 )
+		if( selectedComputerControlInterfaces.size() > 1 )
 		{
 			QMessageBox::critical( master.mainWindow(), feature.name(),
 								   tr( "Please select only one user screen to share.") );
@@ -175,7 +212,7 @@ bool DemoFeaturePlugin::startFeature( VeyonMasterInterface& master, const Featur
 		auto vncServerPortOffset = 0;
 		auto demoServerPort = VeyonCore::config().demoServerPort();
 
-		const auto demoServerInterface = master.selectedComputerControlInterfaces().first();
+		const auto& demoServerInterface = selectedComputerControlInterfaces.constFirst();
 		const auto demoServerHost = demoServerInterface->computer().hostAddress();
 		const auto primaryServerPort = HostAddress::parsePortNumber( demoServerHost );
 
@@ -186,14 +223,6 @@ bool DemoFeaturePlugin::startFeature( VeyonMasterInterface& master, const Featur
 			demoServerPort += sessionId;
 		}
 
-		// start demo server
-		controlFeature( m_demoServerFeature.uid(), Operation::Start,
-						{
-							{ argToString(Argument::VncServerPortOffset), vncServerPortOffset },
-							{ argToString(Argument::DemoServerPort), demoServerPort },
-						},
-						master.selectedComputerControlInterfaces() );
-
 		// start demo clients
 		auto userDemoControlInterfaces = computerControlInterfaces;
 		userDemoControlInterfaces.removeAll( demoServerInterface );
@@ -210,6 +239,14 @@ bool DemoFeaturePlugin::startFeature( VeyonMasterInterface& master, const Featur
 		controlFeature( m_demoClientWindowFeature.uid(), Operation::Start, demoClientArgs,
 						{ master.localSessionControlInterface().weakPointer() } );
 
+		// start demo server
+		controlFeature( m_demoServerFeature.uid(), Operation::Start,
+						{
+							{ argToString(Argument::VncServerPortOffset), vncServerPortOffset },
+							{ argToString(Argument::DemoServerPort), demoServerPort },
+							},
+						selectedComputerControlInterfaces );
+
 		return true;
 	}
 
@@ -233,24 +270,24 @@ bool DemoFeaturePlugin::stopFeature( VeyonMasterInterface& master, const Feature
 		feature == m_shareOwnScreenWindowFeature || feature == m_shareOwnScreenFullScreenFeature ||
 		feature == m_shareUserScreenWindowFeature || feature == m_shareUserScreenFullScreenFeature )
 	{
+		controlFeature( m_demoClientFullScreenFeature.uid(), Operation::Stop, {}, computerControlInterfaces );
 		controlFeature( m_demoClientWindowFeature.uid(), Operation::Stop, {}, computerControlInterfaces );
 
 		controlFeature( m_demoClientWindowFeature.uid(), Operation::Stop, {},
 						{ master.localSessionControlInterface().weakPointer() } );
 
 		// no demo clients left?
-		if( m_demoClientHosts.isEmpty() )
+		if( m_demoServerClients.isEmpty() )
 		{
 			// then we can stop the server
-			controlFeature( m_demoServerFeature.uid(), Operation::Stop, {},
-							{ master.localSessionControlInterface().weakPointer() } );
-
-			controlFeature( m_demoServerFeature.uid(), Operation::Stop, {}, computerControlInterfaces );
+			m_demoServerControlTimer.stop();
 
 			// reset demo access token
 			m_demoAccessToken = CryptoCore::generateChallenge();
 		}
 
+		controlDemoServer();
+
 		return true;
 	}
 
@@ -303,6 +340,12 @@ bool DemoFeaturePlugin::handleFeatureMessage( VeyonServerInterface& server,
 			return true;
 		}
 
+		if( VeyonCore::platform().sessionFunctions().currentSessionHasUser() == false )
+		{
+			vDebug() << "not starting demo client since not running in a user session";
+			return true;
+		}
+
 		auto socket = qobject_cast<QTcpSocket *>( messageContext.ioDevice() );
 		if( socket == nullptr )
 		{
@@ -353,7 +396,10 @@ bool DemoFeaturePlugin::handleFeatureMessage( VeyonWorkerInterface& worker, cons
 			return true;
 
 		case StopDemoServer:
-			delete m_demoServer;
+			if( m_demoServer )
+			{
+				m_demoServer->terminate();
+			}
 			m_demoServer = nullptr;
 
 			QCoreApplication::quit();
@@ -450,10 +496,10 @@ void DemoFeaturePlugin::updateFeatures()
 	{
 		m_screenSelectionFeatures.reserve( m_screens.size() + 1 );
 
-		auto allScreensFlags = Feature::Option | Feature::Master;
+		auto allScreensFlags = Feature::Flag::Option | Feature::Flag::Master;
 		if( m_screenSelection <= ScreenSelectionNone )
 		{
-			allScreensFlags |= Feature::Checked;
+			allScreensFlags |= Feature::Flag::Checked;
 		}
 
 		m_screenSelectionFeatures.append( Feature{ QStringLiteral("DemoAllScreens"),
@@ -465,26 +511,17 @@ void DemoFeaturePlugin::updateFeatures()
 		for( auto screen : qAsConst(m_screens) )
 		{
 			const auto name = QStringLiteral( "DemoScreen%1" ).arg( index );
+			const auto screenName = VeyonCore::screenName(*screen, index);
 
-			auto displayName = tr( "Screen %1 [%2]" ).arg( index ).arg( screen->name() );
-
-#if QT_VERSION >= QT_VERSION_CHECK(5, 9, 0)
-			if( screen->manufacturer().isEmpty() == false &&
-				screen->model().isEmpty() == false )
-			{
-				displayName += QStringLiteral(" – %1 %2").arg( screen->manufacturer(), screen->model() );
-			}
-#endif
-
-			auto flags = Feature::Option | Feature::Master;
+			auto flags = Feature::Flag::Option | Feature::Flag::Master;
 			if( index == m_screenSelection )
 			{
-				flags |= Feature::Checked;
+				flags |= Feature::Flag::Checked;
 			}
 
 			m_screenSelectionFeatures.append( Feature{ name, flags,
 													   Feature::Uid::createUuidV5( m_demoFeature.uid(), name ),
-													   m_demoFeature.uid(), displayName, {}, {}, {} } );
+													   m_demoFeature.uid(), screenName, {}, {}, {} } );
 			++index;
 		}
 	}
@@ -526,36 +563,30 @@ QRect DemoFeaturePlugin::viewportFromScreenSelection() const
 
 
 
-bool DemoFeaturePlugin::controlDemoServer( Operation operation, const QVariantMap& arguments,
-										  const ComputerControlInterfaceList& computerControlInterfaces )
+bool DemoFeaturePlugin::controlDemoServer()
 {
-	if( operation == Operation::Start )
+	if( m_demoServerControlTimer.isActive() )
 	{
-		const auto demoServerPort = arguments.value( argToString(Argument::DemoServerPort),
-													 VeyonCore::config().demoServerPort() + VeyonCore::sessionId() ).toInt();
-		const auto vncServerPortOffset = arguments.value( argToString(Argument::VncServerPortOffset),
-														  VeyonCore::sessionId() ).toInt();
-		const auto demoAccessToken = arguments.value( argToString(Argument::DemoAccessToken),
-													  m_demoAccessToken.toByteArray() ).toByteArray();
+		const auto demoServerPort = m_demoServerArguments.value( argToString(Argument::DemoServerPort),
+																 VeyonCore::config().demoServerPort() + VeyonCore::sessionId() ).toInt();
+		const auto vncServerPortOffset = m_demoServerArguments.value( argToString(Argument::VncServerPortOffset),
+																	  VeyonCore::sessionId() ).toInt();
+		const auto demoAccessToken = m_demoServerArguments.value( argToString(Argument::DemoAccessToken),
+																  m_demoAccessToken.toByteArray() ).toByteArray();
 
 		sendFeatureMessage( FeatureMessage{ m_demoServerFeature.uid(), StartDemoServer }
 								.addArgument( Argument::DemoAccessToken, demoAccessToken )
 								.addArgument( Argument::VncServerPortOffset, vncServerPortOffset )
 								.addArgument( Argument::DemoServerPort, demoServerPort ),
-							computerControlInterfaces );
-
-		return true;
+							m_demoServerControlInterfaces );
 	}
-
-	if( operation == Operation::Stop )
+	else
 	{
 		sendFeatureMessage( FeatureMessage{ m_demoServerFeature.uid(), StopDemoServer },
-							computerControlInterfaces );
-
-		return true;
+							m_demoServerControlInterfaces );
 	}
 
-	return false;
+	return true;
 }
 
 
@@ -587,7 +618,8 @@ bool DemoFeaturePlugin::controlDemoClient( Feature::Uid featureUid, Operation op
 
 		for( const auto& computerControlInterface : computerControlInterfaces )
 		{
-			m_demoClientHosts += computerControlInterface->computer().hostAddress();
+			m_demoServerClients += computerControlInterface;
+
 			if( disableUpdates )
 			{
 				computerControlInterface->setUpdateMode( ComputerControlInterface::UpdateMode::Disabled );
@@ -610,8 +642,10 @@ bool DemoFeaturePlugin::controlDemoClient( Feature::Uid featureUid, Operation op
 
 		for( const auto& computerControlInterface : computerControlInterfaces )
 		{
-			m_demoClientHosts.removeAll( computerControlInterface->computer().hostAddress() );
-			if( enableUpdates )
+			m_demoServerClients.removeAll( computerControlInterface );
+
+			if (enableUpdates &&
+				computerControlInterface->updateMode() == ComputerControlInterface::UpdateMode::Disabled)
 			{
 				computerControlInterface->setUpdateMode( ComputerControlInterface::UpdateMode::Monitoring );
 			}
