@@ -1,7 +1,7 @@
 /*
  * LdapPlugin.h - declaration of LdapPlugin class
  *
- * Copyright (c) 2017-2021 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2017-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -51,7 +51,7 @@ public:
 
 	Plugin::Uid uid() const override
 	{
-		return QStringLiteral("6f0a491e-c1c6-4338-8244-f823b0bf8670");
+		return Plugin::Uid{ QStringLiteral("6f0a491e-c1c6-4338-8244-f823b0bf8670") };
 	}
 
 	QVersionNumber version() const override
