@@ -1,7 +1,7 @@
 /*
  * LdapDirectory.cpp - class representing the LDAP directory and providing access to directory entries
  *
- * Copyright (c) 2016-2021 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2016-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -43,6 +43,14 @@ LdapDirectory::LdapDirectory( const LdapConfiguration& configuration, QObject* p
 
 	m_userLoginNameAttribute = m_configuration.userLoginNameAttribute();
 	m_groupMemberAttribute = m_configuration.groupMemberAttribute();
+
+	m_groupMemberFilterAttribute = m_groupMemberAttribute;
+	if( m_configuration.queryNestedUserGroups() &&
+		m_groupMemberFilterAttribute.contains( QLatin1Char(':') ) == false )
+	{
+		m_groupMemberFilterAttribute.append( QLatin1String(":1.2.840.113556.1.4.1941:") );
+	}
+
 	m_computerDisplayNameAttribute = m_configuration.computerDisplayNameAttribute();
 	m_computerHostNameAttribute = m_configuration.computerHostNameAttribute();
 	m_computerHostNameAsFQDN = m_configuration.computerHostNameAsFQDN();
@@ -51,12 +59,12 @@ LdapDirectory::LdapDirectory( const LdapConfiguration& configuration, QObject* p
 
 	if( m_computerDisplayNameAttribute.isEmpty() )
 	{
-		m_computerDisplayNameAttribute = QStringLiteral("cn");
+		m_computerDisplayNameAttribute = LdapClient::cn();
 	}
 
 	if( m_locationNameAttribute.isEmpty() )
 	{
-		m_locationNameAttribute = QStringLiteral("cn");
+		m_locationNameAttribute = LdapClient::cn();
 	}
 
 	m_usersFilter = m_configuration.usersFilter();
@@ -178,7 +186,7 @@ QStringList LdapDirectory::users( const QString& filterValue )
 QStringList LdapDirectory::groups( const QString& filterValue )
 {
 	return m_client.queryDistinguishedNames( groupsDn(),
-											 LdapClient::constructQueryFilter( QStringLiteral( "cn" ), filterValue ),
+											 LdapClient::constructQueryFilter( LdapClient::cn(), filterValue ),
 											 m_defaultSearchScope );
 }
 
@@ -187,7 +195,7 @@ QStringList LdapDirectory::groups( const QString& filterValue )
 QStringList LdapDirectory::userGroups( const QString& filterValue )
 {
 	return m_client.queryDistinguishedNames( groupsDn(),
-											 LdapClient::constructQueryFilter( QStringLiteral( "cn" ), filterValue, m_userGroupsFilter ),
+											 LdapClient::constructQueryFilter( LdapClient::cn(), filterValue, m_userGroupsFilter ),
 											 m_defaultSearchScope );
 }
 
@@ -270,13 +278,13 @@ QStringList LdapDirectory::groupMembers( const QString& groupDn )
 QStringList LdapDirectory::groupsOfUser( const QString& userDn )
 {
 	const auto userId = groupMemberUserIdentification( userDn );
-	if( m_groupMemberAttribute.isEmpty() || userId.isEmpty() )
+	if( m_groupMemberFilterAttribute.isEmpty() || userId.isEmpty() )
 	{
 		return {};
 	}
 
 	return m_client.queryDistinguishedNames( groupsDn(),
-											 LdapClient::constructQueryFilter( m_groupMemberAttribute, userId, m_userGroupsFilter ),
+											 LdapClient::constructQueryFilter( m_groupMemberFilterAttribute, userId, m_userGroupsFilter ),
 											 m_defaultSearchScope );
 }
 
@@ -285,13 +293,13 @@ QStringList LdapDirectory::groupsOfUser( const QString& userDn )
 QStringList LdapDirectory::groupsOfComputer( const QString& computerDn )
 {
 	const auto computerId = groupMemberComputerIdentification( computerDn );
-	if( m_groupMemberAttribute.isEmpty() || computerId.isEmpty() )
+	if( m_groupMemberFilterAttribute.isEmpty() || computerId.isEmpty() )
 	{
 		return {};
 	}
 
 	return m_client.queryDistinguishedNames( computerGroupsDn(),
-											 LdapClient::constructQueryFilter( m_groupMemberAttribute, computerId, m_computerGroupsFilter ),
+											 LdapClient::constructQueryFilter( m_groupMemberFilterAttribute, computerId, m_computerGroupsFilter ),
 											 m_defaultSearchScope );
 }
 
@@ -309,14 +317,14 @@ QStringList LdapDirectory::locationsOfComputer( const QString& computerDn )
 	}
 
 	const auto computerId = groupMemberComputerIdentification( computerDn );
-	if( m_groupMemberAttribute.isEmpty() || computerId.isEmpty() )
+	if( m_groupMemberFilterAttribute.isEmpty() || computerId.isEmpty() )
 	{
 		return {};
 	}
 
 	return m_client.queryAttributeValues( computerGroupsDn(),
 										  m_locationNameAttribute,
-										  LdapClient::constructQueryFilter( m_groupMemberAttribute, computerId, m_computerGroupsFilter ),
+										  LdapClient::constructQueryFilter( m_groupMemberFilterAttribute, computerId, m_computerGroupsFilter ),
 										  m_defaultSearchScope );
 }
 
