@@ -1,7 +1,7 @@
 /*
  * LdapNetworkObjectDirectory.cpp - provides a NetworkObjectDirectory for LDAP
  *
- * Copyright (c) 2017-2021 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2017-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -174,7 +174,7 @@ NetworkObjectList LdapNetworkObjectDirectory::queryHosts( NetworkObject::Attribu
 	NetworkObjectList hostObjects;
 	hostObjects.reserve( computers.size() );
 
-	for( const auto& computer : computers )
+	for( const auto& computer : qAsConst(computers) )
 	{
 		const auto hostObject = computerToObject( &m_ldapDirectory, computer );
 		if( hostObject.isValid() )
@@ -193,13 +193,13 @@ NetworkObject LdapNetworkObjectDirectory::computerToObject( LdapDirectory* direc
 	auto displayNameAttribute = directory->computerDisplayNameAttribute();
 	if( displayNameAttribute.isEmpty() )
 	{
-		displayNameAttribute = QStringLiteral("cn");
+		displayNameAttribute = LdapClient::cn();
 	}
 
 	auto hostNameAttribute = directory->computerHostNameAttribute();
 	if( hostNameAttribute.isEmpty() )
 	{
-		hostNameAttribute = QStringLiteral("cn");
+		hostNameAttribute = LdapClient::cn();
 	}
 
 	QStringList computerAttributes{ displayNameAttribute, hostNameAttribute };
