@@ -1,7 +1,7 @@
 /*
  * LinuxCoreFunctions.cpp - implementation of LinuxCoreFunctions class
  *
- * Copyright (c) 2017-2021 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2017-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -22,14 +22,17 @@
  *
  */
 
+#include <QElapsedTimer>
 #include <QFileInfo>
 #include <QDBusPendingCall>
 #include <QProcess>
 #include <QProcessEnvironment>
+#include <QScreen>
 #include <QStandardPaths>
 #include <QWidget>
 
 #include <unistd.h>
+#include <proc/readproc.h>
 
 #include "LinuxCoreFunctions.h"
 #include "LinuxDesktopIntegration.h"
@@ -64,28 +67,15 @@ void LinuxCoreFunctions::writeToNativeLoggingSystem( const QString& message, Log
 
 void LinuxCoreFunctions::reboot()
 {
-	systemdLoginManager()->asyncCall( QStringLiteral("Reboot"), false );
-	consoleKitManager()->asyncCall( QStringLiteral("Restart") );
-
-	if( isRunningAsAdmin() )
+	if( systemdLoginManager()->call( QStringLiteral("Reboot"), false ).type() != QDBusMessage::ReplyMessage &&
+		consoleKitManager()->call( QStringLiteral("Restart") ).type() != QDBusMessage::ReplyMessage )
 	{
-		for( const auto& file : { QStringLiteral("/sbin/reboot"), QStringLiteral("/usr/sbin/reboot") } )
-		{
-			if( QFileInfo::exists( file ) )
-			{
-				QProcess::startDetached( file, {} );
-				return;
-			}
-		}
+		prepareSessionBusAccess();
 
-		QProcess::startDetached( QStringLiteral("reboot"), {} );
-	}
-	else
-	{
 		kdeSessionManager()->asyncCall( QStringLiteral("logout"),
-										static_cast<int>( LinuxDesktopIntegration::KDE::ShutdownConfirmNo ),
-										static_cast<int>( LinuxDesktopIntegration::KDE::ShutdownTypeReboot ),
-										static_cast<int>( LinuxDesktopIntegration::KDE::ShutdownModeForceNow ) );
+										LinuxDesktopIntegration::KDE::ShutdownConfirmNo,
+										LinuxDesktopIntegration::KDE::ShutdownTypeReboot,
+										LinuxDesktopIntegration::KDE::ShutdownModeForceNow );
 		gnomeSessionManager()->asyncCall( QStringLiteral("RequestReboot") );
 		mateSessionManager()->asyncCall( QStringLiteral("RequestReboot") );
 		xfcePowerManager()->asyncCall( QStringLiteral("Reboot") );
@@ -98,28 +88,15 @@ void LinuxCoreFunctions::powerDown( bool installUpdates )
 {
 	Q_UNUSED(installUpdates)
 
-	systemdLoginManager()->asyncCall( QStringLiteral("PowerOff"), false );
-	consoleKitManager()->asyncCall( QStringLiteral("Stop") );
-
-	if( isRunningAsAdmin() )
+	if( systemdLoginManager()->call( QStringLiteral("PowerOff"), false ).type() != QDBusMessage::ReplyMessage &&
+		consoleKitManager()->call( QStringLiteral("Stop") ).type() != QDBusMessage::ReplyMessage )
 	{
-		for( const auto& file : { QStringLiteral("/sbin/poweroff"), QStringLiteral("/usr/sbin/poweroff") } )
-		{
-			if( QFileInfo::exists( file ) )
-			{
-				QProcess::startDetached( file, {} );
-				return;
-			}
-		}
+		prepareSessionBusAccess();
 
-		QProcess::startDetached( QStringLiteral("poweroff"), {} );
-	}
-	else
-	{
 		kdeSessionManager()->asyncCall( QStringLiteral("logout"),
-										static_cast<int>( LinuxDesktopIntegration::KDE::ShutdownConfirmNo ),
-										static_cast<int>( LinuxDesktopIntegration::KDE::ShutdownTypeHalt ),
-										static_cast<int>( LinuxDesktopIntegration::KDE::ShutdownModeForceNow ) );
+										LinuxDesktopIntegration::KDE::ShutdownConfirmNo,
+										LinuxDesktopIntegration::KDE::ShutdownTypeHalt,
+										LinuxDesktopIntegration::KDE::ShutdownModeForceNow );
 		gnomeSessionManager()->asyncCall( QStringLiteral("RequestShutdown") );
 		mateSessionManager()->asyncCall( QStringLiteral("RequestShutdown") );
 		xfcePowerManager()->asyncCall( QStringLiteral("Shutdown") );
@@ -173,7 +150,7 @@ void LinuxCoreFunctions::disableScreenSaver()
 		DPMSGetTimeouts( display, &m_dpmsStandbyTimeout, &m_dpmsSuspendTimeout, &m_dpmsOffTimeout );
 		DPMSSetTimeouts( display, 0, 0, 0 );
 	}
-	else
+	else if( qEnvironmentVariableIsSet("XRDP_SESSION") == false )
 	{
 		vWarning() << "DPMS extension not supported!";
 	}
@@ -249,6 +226,21 @@ bool LinuxCoreFunctions::runProgramAsUser( const QString& program, const QString
 {
 	Q_UNUSED(desktop);
 
+	const auto uid = LinuxUserFunctions::userIdFromName( username );
+	if( uid <= 0 )
+	{
+		return false;
+	}
+
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+	auto process = new QProcess;
+	process->setChildProcessModifier( [uid]() {
+		if( setuid( uid ) != 0 )
+		{
+			qFatal( "Could not set UID for child process!" );
+		};
+	} );
+#else
 	class UserProcess : public QProcess // clazy:exclude=missing-qobject-macro
 	{
 	public:
@@ -270,13 +262,9 @@ bool LinuxCoreFunctions::runProgramAsUser( const QString& program, const QString
 		const uid_t m_uid;
 	};
 
-	const auto uid = LinuxUserFunctions::userIdFromName( username );
-	if( uid <= 0 )
-	{
-		return false;
-	}
-
 	auto process = new UserProcess( uid );
+#endif
+
 	QObject::connect( process, QOverload<int, QProcess::ExitStatus>::of( &QProcess::finished ), &QProcess::deleteLater );
 	process->start( program, parameters );
 
@@ -292,6 +280,46 @@ QString LinuxCoreFunctions::genericUrlHandler() const
 
 
 
+QString LinuxCoreFunctions::queryDisplayDeviceName(const QScreen& screen) const
+{
+	QStringList nameParts;
+#if QT_VERSION >= QT_VERSION_CHECK(5, 9, 0)
+	nameParts.append(screen.manufacturer());
+	nameParts.append(screen.model());
+#endif
+	nameParts.removeAll({});
+	if(nameParts.isEmpty())
+	{
+		return screen.name();
+	}
+
+	return QStringLiteral("%1 [%2]").arg(nameParts.join(QLatin1Char(' ')), screen.name());
+}
+
+
+
+bool LinuxCoreFunctions::prepareSessionBusAccess()
+{
+	const auto uid = LinuxUserFunctions::userIdFromName( VeyonCore::platform().userFunctions().currentUser() );
+	if( uid > 0 )
+	{
+		if( seteuid(uid) == 0 )
+		{
+			return true;
+		}
+
+		vWarning() << "could not set effective UID - DBus calls on the session bus likely will fail";
+	}
+	else
+	{
+		vWarning() << "could not determine UID of current user - DBus calls on the session bus likely will fail";
+	}
+
+	return false;
+}
+
+
+
 /*! Returns DBus interface for session manager of KDE desktop */
 LinuxCoreFunctions::DBusInterfacePointer LinuxCoreFunctions::kdeSessionManager()
 {
@@ -388,3 +416,55 @@ void LinuxCoreFunctions::restartDisplayManagers()
 		systemctl( { QStringLiteral("restart"), displayManager } );
 	}
 }
+
+
+
+void LinuxCoreFunctions::forEachChildProcess( const std::function<bool(proc_t*)>& visitor,
+											 int parentPid, int flags, bool visitParent )
+{
+	QProcessEnvironment sessionEnv;
+
+	const auto proc = openproc( flags | PROC_FILLSTAT /* required for proc_t::ppid */ );
+	proc_t* procInfo = nullptr;
+
+	QList<int> ppids;
+
+	while( ( procInfo = readproc( proc, nullptr ) ) )
+	{
+		if( procInfo->ppid == parentPid )
+		{
+			if( visitParent == false || visitor( procInfo ) )
+			{
+				ppids.append( procInfo->tid );
+			}
+		}
+		else if( ppids.contains( procInfo->ppid ) && visitor( procInfo ) )
+		{
+			ppids.append( procInfo->tid );
+		}
+
+		freeproc( procInfo );
+	}
+
+	closeproc( proc );
+}
+
+
+
+bool LinuxCoreFunctions::waitForProcess( qint64 pid, int timeout, int sleepInterval )
+{
+	QElapsedTimer timeoutTimer;
+	timeoutTimer.start();
+
+	while( QFileInfo::exists( QStringLiteral("/proc/%1").arg( pid ) ) )
+	{
+		if( timeoutTimer.elapsed() >= timeout )
+		{
+			return false;
+		}
+
+		QThread::msleep( static_cast<uint64_t>( sleepInterval ) );
+	}
+
+	return true;
+}
