@@ -1,7 +1,7 @@
 /*
  * LinuxCoreFunctions.h - declaration of LinuxCoreFunctions class
  *
- * Copyright (c) 2017-2021 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2017-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -29,6 +29,8 @@
 
 #include "PlatformCoreFunctions.h"
 
+struct proc_t;
+
 // clazy:excludeall=copyable-polymorphic
 
 class LinuxCoreFunctions : public PlatformCoreFunctions
@@ -62,8 +64,11 @@ public:
 
 	QString genericUrlHandler() const override;
 
+	QString queryDisplayDeviceName(const QScreen& screen) const override;
+
 	using DBusInterfacePointer = QSharedPointer<QDBusInterface>;
 
+	static bool prepareSessionBusAccess();
 	static DBusInterfacePointer kdeSessionManager();
 	static DBusInterfacePointer gnomeSessionManager();
 	static DBusInterfacePointer mateSessionManager();
@@ -75,6 +80,11 @@ public:
 
 	static void restartDisplayManagers();
 
+	static void forEachChildProcess( const std::function<bool(proc_t *)>& visitor,
+							 int parentPid, int flags, bool visitParent );
+
+	static bool waitForProcess( qint64 pid, int timeout, int sleepInterval );
+
 private:
 	int m_screenSaverTimeout{0};
 	int m_screenSaverPreferBlanking{0};
