@@ -1,7 +1,7 @@
 /*
  * LinuxServiceFunctions.cpp - implementation of LinuxServiceFunctions class
  *
- * Copyright (c) 2017-2021 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2017-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -25,17 +25,12 @@
 #include <QDBusReply>
 #include <QEventLoop>
 #include <QFileInfo>
-#include <QProcess>
 #include <QTimer>
 
-#include <csignal>
-#include <sys/types.h>
-
-#include "Filesystem.h"
-#include "LinuxCoreFunctions.h"
+#include "LinuxPlatformConfiguration.h"
+#include "LinuxServerProcess.h"
 #include "LinuxServiceCore.h"
 #include "LinuxSessionFunctions.h"
-#include "ProcessHelper.h"
 #include "VeyonConfiguration.h"
 
 
@@ -56,12 +51,7 @@ LinuxServiceCore::~LinuxServiceCore()
 
 void LinuxServiceCore::run()
 {
-	const auto sessions = listSessions();
-
-	for( const auto& s : sessions )
-	{
-		startServer( s, QDBusObjectPath( s ) );
-	}
+	startServers();
 
 	QEventLoop eventLoop;
 	eventLoop.exec();
@@ -71,108 +61,13 @@ void LinuxServiceCore::run()
 
 void LinuxServiceCore::startServer( const QString& login1SessionId, const QDBusObjectPath& sessionObjectPath )
 {
+	Q_UNUSED(login1SessionId)
+
 	const auto sessionPath = sessionObjectPath.path();
 
-	const auto sessionType = LinuxSessionFunctions::getSessionType( sessionPath );
+	vDebug() << "new session" << sessionPath;
 
-	if( sessionType == QLatin1String("wayland") )
-	{
-		vCritical() << "Can't start Veyon Server in Wayland sessions as this is not yet supported. Please switch to X11-based sessions!";
-		return;
-	}
-
-	// do not start server for non-graphical sessions
-	if( sessionType != QLatin1String("x11") )
-	{
-		return;
-	}
-
-	const auto sessionState = LinuxSessionFunctions::getSessionState( sessionPath );
-	if( sessionState == LinuxSessionFunctions::State::Opening )
-	{
-		vDebug() << "Session" << sessionPath << "still opening - retrying in" << SessionStateProbingInterval << "msecs";
-		QTimer::singleShot( SessionStateProbingInterval, this, [=]() { startServer( login1SessionId, sessionObjectPath ); } );
-		return;
-	}
-
-	// only start server for online or active sessions
-	if( sessionState != LinuxSessionFunctions::State::Online &&
-		sessionState != LinuxSessionFunctions::State::Active )
-	{
-		vDebug() << "Not starting server for session" << sessionPath << "in state" << sessionState;
-		return;
-	}
-
-	const auto sessionLeader = LinuxSessionFunctions::getSessionLeaderPid( sessionPath );
-	if( sessionLeader < 0 )
-	{
-		vCritical() << "No leader available for session" << sessionPath;
-		return;
-	}
-
-	auto sessionEnvironment = LinuxSessionFunctions::getSessionEnvironment( sessionLeader );
-
-	if( sessionEnvironment.isEmpty() )
-	{
-		vWarning() << "Environment for session" << sessionPath << "not yet available - retrying in"
-				   << SessionEnvironmentProbingInterval << "msecs";
-		QTimer::singleShot( SessionEnvironmentProbingInterval, this,
-							[=]() { startServer( login1SessionId, sessionObjectPath ); } );
-		return;
-	}
-
-	if( m_sessionManager.multiSession() == false && m_serverProcesses.isEmpty() == false )
-	{
-		// make sure no other server is still running
-		stopAllServers();
-	}
-
-	const auto sessionUptime = LinuxSessionFunctions::getSessionUptimeSeconds( sessionPath );
-
-	if( sessionUptime >= 0 && sessionUptime < SessionUptimeSecondsMinimum )
-	{
-		vDebug() << "Session" << sessionPath << "too young - retrying in" << SessionUptimeProbingInterval << "msecs";
-		QTimer::singleShot( SessionUptimeProbingInterval, this, [=]() { startServer( login1SessionId, sessionObjectPath ); } );
-		return;
-	}
-
-	const auto seat = LinuxSessionFunctions::getSessionSeat( sessionPath );
-	const auto sessionId = LinuxSessionFunctions::getSessionId( sessionPath );
-
-	// if pam-systemd is not in use, we have to set the XDG_SESSION_ID environment variable manually
-	if( sessionEnvironment.contains( LinuxSessionFunctions::xdgSessionIdEnvVarName() ) == false )
-	{
-		sessionEnvironment.insert( LinuxSessionFunctions::xdgSessionIdEnvVarName(), sessionId );
-	}
-
-	const auto veyonSessionId = m_sessionManager.openSession( sessionId );
-
-	vInfo() << "Starting server for new" << qUtf8Printable(sessionType) << "session" << sessionPath
-			<< "with ID" << veyonSessionId
-			<< "at seat" << seat.path;
-
-	sessionEnvironment.insert( QLatin1String( ServiceDataManager::serviceDataTokenEnvironmentVariable() ),
-							   QString::fromUtf8( m_dataManager.token().toByteArray() ) );
-
-	auto process = new QProcess( this );
-	process->setProcessEnvironment( sessionEnvironment );
-
-	if( VeyonCore::config().logToSystem() )
-	{
-		process->setProcessChannelMode( QProcess::ForwardedChannels );
-	}
-
-	const auto catchsegv{ QStringLiteral("/usr/bin/catchsegv") };
-	if( VeyonCore::isDebugging() && QFileInfo::exists( catchsegv ) )
-	{
-		process->start( catchsegv, { VeyonCore::filesystem().serverFilePath() } );
-	}
-	else
-	{
-		process->start( VeyonCore::filesystem().serverFilePath(), QStringList{} );
-	}
-
-	m_serverProcesses[sessionPath] = process;
+	startServer( sessionPath );
 }
 
 
@@ -183,9 +78,17 @@ void LinuxServiceCore::stopServer( const QString& login1SessionId, const QDBusOb
 
 	const auto sessionPath = sessionObjectPath.path();
 
+	vDebug() << "session removed" << sessionPath;
+
 	if( m_serverProcesses.contains( sessionPath ) )
 	{
 		stopServer( sessionPath );
+
+		// make sure to (re-)start server instances for preempted/suspended sessions such as the login manager session
+		if( m_sessionManager.mode() != PlatformSessionManager::Mode::Multi )
+		{
+			startServers();
+		}
 	}
 }
 
@@ -218,9 +121,146 @@ void LinuxServiceCore::connectToLoginManager()
 
 
 
+void LinuxServiceCore::startServers()
+{
+	vDebug();
+
+	const auto sessions = LinuxSessionFunctions::listSessions();
+
+	for( const auto& s : sessions )
+	{
+		if( m_serverProcesses.contains( s ) == false &&
+			m_deferredServerSessions.contains( s ) == false &&
+			( m_sessionManager.mode() == PlatformSessionManager::Mode::Multi || m_serverProcesses.isEmpty() ) )
+		{
+			startServer( s );
+		}
+	}
+}
+
+
+
+void LinuxServiceCore::startServer( const QString& sessionPath )
+{
+	const auto sessionType = LinuxSessionFunctions::getSessionType( sessionPath );
+
+	if( sessionType == LinuxSessionFunctions::Type::Wayland )
+	{
+		vWarning() << "Wayland session detected but trying to start Veyon Server anyway, even though Veyon Server does "
+					  "not supported Wayland sessions. If you encounter problems, please switch to X11-based sessions!";
+	}
+
+	// do not start server for non-graphical sessions
+	if( sessionType == LinuxSessionFunctions::Type::TTY )
+	{
+		vDebug() << "Not starting Veyon Server in TTY session";
+		return;
+	}
+
+	// do not start server for sessions with unspecified type
+	if (sessionType == LinuxSessionFunctions::Type::Unspecified)
+	{
+		vDebug() << "Not starting Veyon Server in a session with unspecified type";
+		return;
+	}
+
+	const auto sessionState = LinuxSessionFunctions::getSessionState( sessionPath );
+	if( sessionState == LinuxSessionFunctions::State::Opening )
+	{
+		vDebug() << "Session" << sessionPath << "still is being opening - retrying in" << SessionStateProbingInterval << "msecs";
+		deferServerStart( sessionPath, SessionStateProbingInterval );
+		return;
+	}
+
+	// only start server for online or active sessions
+	if( sessionState != LinuxSessionFunctions::State::Online &&
+		sessionState != LinuxSessionFunctions::State::Active )
+	{
+		vInfo() << "Not starting server for session" << sessionPath << "in state" << sessionState;
+		return;
+	}
+
+	const auto sessionLeader = LinuxSessionFunctions::getSessionLeaderPid( sessionPath );
+	if( sessionLeader < 0 )
+	{
+		vCritical() << "No leader available for session" << sessionPath;
+		return;
+	}
+
+	auto sessionEnvironment = LinuxSessionFunctions::getSessionEnvironment( sessionLeader );
+
+	if( sessionEnvironment.isEmpty() )
+	{
+		vWarning() << "Environment for session" << sessionPath << "not yet available - retrying in"
+				   << SessionEnvironmentProbingInterval << "msecs";
+		deferServerStart( sessionPath, SessionEnvironmentProbingInterval );
+		return;
+	}
+
+	if( m_sessionManager.mode() != PlatformSessionManager::Mode::Multi )
+	{
+		// make sure no other server is still running
+		stopAllServers();
+	}
+
+	const auto sessionUptime = LinuxSessionFunctions::getSessionUptimeSeconds( sessionPath );
+	const auto minimumSessionUptime = LinuxPlatformConfiguration(&VeyonCore::config()).minimumUserSessionLifetime();
+
+	if( sessionUptime >= 0 &&
+		sessionUptime < minimumSessionUptime )
+	{
+		vDebug() << "Session" << sessionPath << "too young - retrying in" << minimumSessionUptime - sessionUptime << "msecs";
+		deferServerStart( sessionPath, int(minimumSessionUptime - sessionUptime) );
+		return;
+	}
+
+	sessionEnvironment.insert( LinuxSessionFunctions::sessionPathEnvVarName(), sessionPath );
+
+	// if pam-systemd is not in use, we have to set the XDG_SESSION_ID environment variable manually
+	if( sessionEnvironment.contains( LinuxSessionFunctions::xdgSessionIdEnvVarName() ) == false )
+	{
+		const auto sessionId = LinuxSessionFunctions::getSessionId(sessionPath);
+		if (sessionId.isEmpty() == false)
+		{
+			sessionEnvironment.insert(LinuxSessionFunctions::xdgSessionIdEnvVarName(), sessionId);
+		}
+	}
+
+	const auto sessionId = m_sessionManager.openSession( sessionPath );
+
+	vInfo() << "Starting server for new session" << sessionPath
+			<< "with ID" << sessionId
+			<< "at seat" << LinuxSessionFunctions::getSessionSeat( sessionPath ).path;
+
+	sessionEnvironment.insert( QLatin1String( ServiceDataManager::serviceDataTokenEnvironmentVariable() ),
+							   QString::fromUtf8( m_dataManager.token().toByteArray() ) );
+
+	auto serverProcess = new LinuxServerProcess( sessionEnvironment, sessionPath, sessionId, this );
+	serverProcess->start();
+
+	connect( serverProcess, &QProcess::stateChanged, this, [=]() { checkSessionState( sessionPath ); } );
+
+	m_serverProcesses[sessionPath] = serverProcess;
+	m_deferredServerSessions.removeAll( sessionPath );
+}
+
+
+
+void LinuxServiceCore::deferServerStart( const QString& sessionPath, int delay )
+{
+	QTimer::singleShot( delay, this, [=]() { startServer( sessionPath ); } );
+
+	if( m_deferredServerSessions.contains( sessionPath ) == false )
+	{
+		m_deferredServerSessions.append( sessionPath );
+	}
+}
+
+
+
 void LinuxServiceCore::stopServer( const QString& sessionPath )
 {
-	m_sessionManager.closeSession( LinuxSessionFunctions::getSessionId( sessionPath ) );
+	m_sessionManager.closeSession( sessionPath );
 
 	if( m_serverProcesses.contains( sessionPath ) == false )
 	{
@@ -229,24 +269,11 @@ void LinuxServiceCore::stopServer( const QString& sessionPath )
 
 	vInfo() << "stopping server for removed session" << sessionPath;
 
-	auto process = qAsConst(m_serverProcesses)[sessionPath];
+	auto serverProcess = qAsConst(m_serverProcesses)[sessionPath];
+	serverProcess->disconnect(this);
+	serverProcess->stop();
+	serverProcess->deleteLater();
 
-	// tell x11vnc to shutdown
-	kill( pid_t(process->processId()), SIGINT );
-
-	if( ProcessHelper::waitForProcess( process, ServerShutdownTimeout, ServerWaitSleepInterval ) == false )
-	{
-		process->terminate();
-
-		if( ProcessHelper::waitForProcess( process, ServerTerminateTimeout, ServerWaitSleepInterval ) == false )
-		{
-			vWarning() << "server for session" << sessionPath << "still running - killing now";
-			process->kill();
-			ProcessHelper::waitForProcess( process, ServerKillTimeout, ServerWaitSleepInterval );
-		}
-	}
-
-	delete process;
 	m_serverProcesses.remove( sessionPath );
 }
 
@@ -262,32 +289,13 @@ void LinuxServiceCore::stopAllServers()
 
 
 
-QStringList LinuxServiceCore::listSessions()
+void LinuxServiceCore::checkSessionState( const QString& sessionPath )
 {
-	QStringList sessions;
-
-	const QDBusReply<QDBusArgument> reply = m_loginManager->call( QStringLiteral("ListSessions") );
-
-	if( reply.isValid() )
+	const auto sessionState = LinuxSessionFunctions::getSessionState( sessionPath );
+	if( sessionState == LinuxSessionFunctions::State::Closing ||
+		sessionState == LinuxSessionFunctions::State::Unknown )
 	{
-		const auto data = reply.value();
-
-		data.beginArray();
-		while( data.atEnd() == false )
-		{
-			LinuxSessionFunctions::LoginDBusSession session;
-
-			data.beginStructure();
-			data >> session.id >> session.uid >> session.name >> session.seatId >> session.path;
-			data.endStructure();
-
-			sessions.append( session.path.path() );
-		}
-		return sessions;
+		vDebug() << "Stopping server for currently closing session" << sessionPath;
+		stopServer( sessionPath );
 	}
-
-	vCritical() << "Could not query sessions:" << reply.error().message();
-
-	return sessions;
 }
-
