@@ -1,7 +1,7 @@
 /*
  * LinuxSessionFunctions.h - declaration of LinuxSessionFunctions class
  *
- * Copyright (c) 2020-2021 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2020-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -24,7 +24,7 @@
 
 #pragma once
 
-#include <QDBusInterface>
+#include <QDBusObjectPath>
 #include <QProcessEnvironment>
 
 #include "PlatformSessionFunctions.h"
@@ -35,7 +35,7 @@ class LinuxSessionFunctions : public PlatformSessionFunctions
 {
 	Q_GADGET
 public:
-	using LoginDBusSession = struct {
+	using LoginDBusSession = struct LoginDBusSession {
 		QString id;
 		quint32 uid{0};
 		QString name;
@@ -60,16 +60,40 @@ public:
 	};
 	Q_ENUM(State)
 
+	enum class Class
+	{
+		Unknown,
+		User,
+		Greeter,
+		LockScreen
+	};
+	Q_ENUM(Class)
+
+	enum class Type
+	{
+		Unspecified,
+		TTY,
+		X11,
+		Mir,
+		Wayland
+	};
+	Q_ENUM(Type)
+
 	SessionId currentSessionId() override;
 
 	QString currentSessionType() const override;
+	bool currentSessionHasUser() const override;
 
-	static QVariant getSessionProperty( const QString& session, const QString& property );
+	static QStringList listSessions();
+
+	static QVariant getSessionProperty(const QString& session, const QString& property, bool logErrors = true);
 
 	static int getSessionLeaderPid( const QString& session );
 	static qint64 getSessionUptimeSeconds( const QString& session );
-	static QString getSessionType( const QString& session );
-	static QString getSessionId( const QString& session );
+	static Class getSessionClass( const QString& session );
+	static Type getSessionType( const QString& session );
+	static QString getSessionId(const QString& session, bool logErrors = true);
+	static QString getSessionUser( const QString& session );
 	static State getSessionState( const QString& session );
 	static LoginDBusSessionSeat getSessionSeat( const QString& session );
 
@@ -82,4 +106,17 @@ public:
 		return QStringLiteral("XDG_SESSION_ID");
 	}
 
+	static QString xdgSessionClassEnvVarName()
+	{
+		return QStringLiteral("XDG_SESSION_CLASS");
+	}
+
+	static QString sessionPathEnvVarName()
+	{
+		return QStringLiteral("VEYON_SESSION_PATH");
+	}
+
+	static bool isOpen( const QString& session );
+	static bool isGraphical( const QString& session );
+
 };
