@@ -1,7 +1,7 @@
 /*
  * LinuxUserFunctions.cpp - implementation of LinuxUserFunctions class
  *
- * Copyright (c) 2017-2021 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2017-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -25,6 +25,7 @@
 #include <QDataStream>
 #include <QDBusReply>
 #include <QProcess>
+#include <QRegularExpression>
 
 #include "LinuxCoreFunctions.h"
 #include "LinuxDesktopIntegration.h"
@@ -37,6 +38,7 @@
 #define XK_MISCELLANY
 
 #include <X11/keysymdef.h>
+#include <X11/Xlib.h>
 
 #include <pwd.h>
 #include <unistd.h>
@@ -44,7 +46,7 @@
 
 QString LinuxUserFunctions::fullName( const QString& username )
 {
-	auto pw_entry = getpwnam( VeyonCore::stripDomain( username ).toUtf8().constData() );
+	const auto pw_entry = getpwnam(username.toUtf8().constData());
 
 	if( pw_entry )
 	{
@@ -177,8 +179,6 @@ QStringList LinuxUserFunctions::groupsOfUser( const QString& username, bool quer
 
 	QStringList groupList;
 
-	const auto strippedUsername = VeyonCore::stripDomain( username );
-
 	QProcess getentProcess;
 	getentProcess.start( QStringLiteral("getent"), { QStringLiteral("group") } );
 	getentProcess.waitForFinished();
@@ -188,7 +188,7 @@ QStringList LinuxUserFunctions::groupsOfUser( const QString& username, bool quer
 	{
 		const auto groupComponents = group.split( QLatin1Char(':') );
 		if( groupComponents.size() == 4 &&
-				groupComponents.last().split( QLatin1Char(',') ).contains( strippedUsername ) )
+			groupComponents.last().split( QLatin1Char(',') ).contains(username))
 		{
 			groupList += groupComponents.first(); // clazy:exclude=reserve-candidates
 		}
@@ -203,24 +203,12 @@ QStringList LinuxUserFunctions::groupsOfUser( const QString& username, bool quer
 
 bool LinuxUserFunctions::isAnyUserLoggedOn()
 {
-	QProcess whoProcess;
-	whoProcess.start( QStringLiteral("who"), QStringList{} );
-	whoProcess.waitForFinished( WhoProcessTimeout );
-
-	if( whoProcess.exitCode() != 0 )
+	const auto sessions = LinuxSessionFunctions::listSessions();
+	for( const auto& session : sessions )
 	{
-		return false;
-	}
-
-	const auto displayManagerUsers = LinuxPlatformConfiguration( &VeyonCore::config() ).displayManagerUsers().
-			split( QLatin1Char(',') );
-
-	const auto lines = whoProcess.readAll().split( '\n' );
-	for( const auto& line : lines )
-	{
-		const auto user = QString::fromUtf8( line.split( ' ' ).value( 0 ) );
-		if( user.isEmpty() == false &&
-			displayManagerUsers.contains( user ) == false )
+		if( LinuxSessionFunctions::isOpen( session ) &&
+			LinuxSessionFunctions::isGraphical( session ) &&
+			LinuxSessionFunctions::getSessionClass( session ) == LinuxSessionFunctions::Class::User )
 		{
 			return true;
 		}
@@ -235,6 +223,16 @@ QString LinuxUserFunctions::currentUser()
 {
 	QString username;
 
+	const auto sessionUserPath = LinuxSessionFunctions::getSessionUser(LinuxSessionFunctions::currentSessionPath());
+	if (sessionUserPath.isEmpty() == false)
+	{
+		username = getUserProperty(sessionUserPath, QStringLiteral("Name")).toString();
+		if (username.isEmpty() == false)
+		{
+			return username;
+		}
+	}
+
 	const auto envUser = qgetenv( "USER" );
 
 	struct passwd * pw_entry = nullptr;
@@ -289,10 +287,51 @@ bool LinuxUserFunctions::performLogon( const QString& username, const Password&
 {
 	LinuxKeyboardInput input;
 
-	input.sendString( username );
-	input.pressAndReleaseKey( XK_Tab );
-	input.sendString( QString::fromUtf8( password.toByteArray() ) );
-	input.pressAndReleaseKey( XK_Return );
+	auto sequence = LinuxPlatformConfiguration( &VeyonCore::config() ).userLoginKeySequence();
+
+	if( sequence.isEmpty() == true )
+	{
+		sequence = QStringLiteral("%username%<Tab>%password%<Return>");
+	}
+
+	auto matchIterator = QRegularExpression( QStringLiteral("(<[\\w\\d_]+>|%username%|%password%|[\\w\\d]+)") )
+							 .globalMatch( sequence );
+	if( matchIterator.hasNext() == false )
+	{
+		vCritical() << "invalid user login key sequence";
+		return false;
+	}
+
+	while( matchIterator.hasNext() )
+	{
+		const auto token = matchIterator.next().captured(0);
+		if( token == QStringLiteral("%username%") )
+		{
+			input.sendString( username );
+		}
+		else if( token == QStringLiteral("%password%") )
+		{
+			input.sendString( QString::fromUtf8( password.toByteArray() ) );
+		}
+		else if( token.startsWith( QLatin1Char('<') ) && token.endsWith( QLatin1Char('>') ) )
+		{
+			const auto keysymString = token.mid( 1, token.length() - 2 );
+			const auto keysym = XStringToKeysym( keysymString.toLatin1().constData() );
+			if( keysym != NoSymbol )
+			{
+				input.pressAndReleaseKey( keysym );
+			}
+			else
+			{
+				vCritical() << "unresolved keysym" << keysymString;
+				return false;
+			}
+		}
+		else if( token.isEmpty() == false )
+		{
+			input.sendString( token );
+		}
+	}
 
 	return true;
 }
@@ -301,15 +340,37 @@ bool LinuxUserFunctions::performLogon( const QString& username, const Password&
 
 void LinuxUserFunctions::logoff()
 {
+	LinuxCoreFunctions::prepareSessionBusAccess();
+
 	// logout via common session managers
-	LinuxCoreFunctions::kdeSessionManager()->asyncCall( QStringLiteral("logout"),
-														static_cast<int>( LinuxDesktopIntegration::KDE::ShutdownConfirmNo ),
-														static_cast<int>( LinuxDesktopIntegration::KDE::ShutdownTypeLogout ),
-														static_cast<int>( LinuxDesktopIntegration::KDE::ShutdownModeForceNow ) );
-	LinuxCoreFunctions::gnomeSessionManager()->asyncCall( QStringLiteral("Logout"),
-														  static_cast<int>( LinuxDesktopIntegration::Gnome::GSM_MANAGER_LOGOUT_MODE_FORCE ) );
-	LinuxCoreFunctions::mateSessionManager()->asyncCall( QStringLiteral("Logout"),
-														 static_cast<int>( LinuxDesktopIntegration::Mate::GSM_LOGOUT_MODE_FORCE ) );
+	// logout via common session managers
+	for( const auto& call : std::initializer_list<std::function<QDBusMessage()>>
+		 {
+			 []() {
+				 return LinuxCoreFunctions::kdeSessionManager()
+					 ->call( QStringLiteral("logout"),
+							 LinuxDesktopIntegration::KDE::ShutdownConfirmNo,
+							 LinuxDesktopIntegration::KDE::ShutdownTypeLogout,
+							 LinuxDesktopIntegration::KDE::ShutdownModeForceNow );
+			 },
+			 []() {
+				 return LinuxCoreFunctions::gnomeSessionManager()
+					 ->call( QStringLiteral("Logout"),
+							 LinuxDesktopIntegration::Gnome::GSM_MANAGER_LOGOUT_MODE_FORCE );
+			 },
+			 []() {
+				 return LinuxCoreFunctions::mateSessionManager()
+					 ->call( QStringLiteral("Logout"),
+							 LinuxDesktopIntegration::Mate::GSM_LOGOUT_MODE_FORCE );
+			 }
+		 } )
+	{
+		// call successful?
+		if( call().type() == QDBusMessage::ReplyMessage )
+		{
+			return;
+		}
+	}
 
 	// Xfce logout
 	QProcess::startDetached( QStringLiteral("xfce4-session-logout --logout"), {} );
@@ -342,7 +403,7 @@ bool LinuxUserFunctions::authenticate( const QString& username, const Password&
 	const auto pamService = LinuxPlatformConfiguration( &VeyonCore::config() ).pamServiceName();
 
 	QDataStream ds( &p );
-	ds << VeyonCore::stripDomain( username ).toUtf8();
+	ds << username.toUtf8();
 	ds << password.toByteArray();
 	ds << pamService.toUtf8();
 
@@ -372,3 +433,30 @@ uid_t LinuxUserFunctions::userIdFromName( const QString& username )
 
 	return 0;
 }
+
+
+
+QVariant LinuxUserFunctions::getUserProperty(const QString& userPath, const QString& property, bool logErrors)
+{
+	QDBusInterface loginManager(QStringLiteral("org.freedesktop.login1"),
+								 userPath,
+								 QStringLiteral("org.freedesktop.DBus.Properties"),
+								 QDBusConnection::systemBus());
+
+	const QDBusReply<QDBusVariant> reply = loginManager.call(QStringLiteral("Get"),
+															  QStringLiteral("org.freedesktop.login1.User"),
+															  property);
+
+	if( reply.isValid() == false )
+	{
+		if (logErrors)
+		{
+			vCritical() << "Could not query property" << property
+						<< "of user" << userPath
+						<< "error:" << reply.error().message();
+		}
+		return {};
+	}
+
+	return reply.value().variant();
+}
