@@ -1,7 +1,7 @@
 /*
  * WebApiController.cpp - implementation of WebApiController class
  *
- * Copyright (c) 2020-2021 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2020-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -27,6 +27,7 @@
 #include <QImageWriter>
 
 #include "ComputerControlInterface.h"
+#include "FeatureManager.h"
 #include "PlatformSessionFunctions.h"
 #include "WebApiAuthenticationProxy.h"
 #include "WebApiConfiguration.h"
@@ -98,11 +99,16 @@ WebApiController::Response WebApiController::getAuthenticationMethods( const Req
 	const auto proxy = new WebApiAuthenticationProxy( m_configuration );
 	proxy->populateCredentials( proxy->dummyAuthenticationMethod(), {} );
 
-	connection.controlInterface()->start( {}, ComputerControlInterface::UpdateMode::Monitoring, proxy );
+	connection.controlInterface()->start( {}, ComputerControlInterface::UpdateMode::Basic, proxy );
 
 	if( proxy->waitForAuthenticationMethods(
 			m_configuration.connectionAuthenticationTimeout() * MillisecondsPerSecond ) == false )
 	{
+		if( proxy->protocolErrorOccurred() )
+		{
+			return Error::ProtocolMismatch;
+		}
+
 		vWarning() << "waiting for authentication methods timed out";
 		return Error::ConnectionTimedOut;
 	}
@@ -150,7 +156,7 @@ WebApiController::Response WebApiController::performAuthentication( const Reques
 	// create connection (including timer resources) in main thread
 	auto connection = runInMainThread<WebApiConnectionPointer>( [host, proxy]() {
 		auto connection = new WebApiConnection{host.isEmpty() ? QStringLiteral("localhost") : host};
-		connection->controlInterface()->start( {}, ComputerControlInterface::UpdateMode::Monitoring, proxy );
+		connection->controlInterface()->start( {}, ComputerControlInterface::UpdateMode::Basic, proxy );
 
 		// make shared pointer destroy the connection in main thread again
 		return WebApiConnectionPointer{ connection,
@@ -165,6 +171,11 @@ WebApiController::Response WebApiController::performAuthentication( const Reques
 		return Error::ConnectionTimedOut;
 	}
 
+	if( proxy->protocolErrorOccurred() )
+	{
+		return Error::ProtocolMismatch;
+	}
+
 	if( proxy->authenticationMethods().contains( methodUuid ) == false )
 	{
 		return Error::AuthenticationMethodNotAvailable;
@@ -224,14 +235,6 @@ WebApiController::Response WebApiController::performAuthentication( const Reques
 			lifetimeTimer->start( connectionLifetime );
 		} );
 
-		connect( connection->controlInterface().data(), &ComputerControlInterface::featureMessageReceived, this,
-				 [this]( const FeatureMessage& featureMessage,
-						 const ComputerControlInterface::Pointer& computerControlInterface ) {
-					 computerControlInterface->lock();
-					 m_featureManager.handleFeatureMessage( computerControlInterface, featureMessage );
-					 computerControlInterface->unlock();
-				 } );
-
 		return QVariantMap{
 			{ connectionUidHeaderFieldName().toLower(), uuidToString( uuid ) },
 			{ k2s(Key::ValidUntil), QDateTime::currentSecsSinceEpoch() + connectionLifetime / MillisecondsPerSecond }
@@ -253,7 +256,7 @@ WebApiController::Response WebApiController::closeConnection( const Request& req
 		return checkResponse;
 	}
 
-	removeConnection( QUuid( request.headers[connectionUidHeaderFieldName()].toString() ) );
+	removeConnection( lookupHeaderField(request.headers, connectionUidHeaderFieldName()).toUuid() );
 
 	return {};
 }
@@ -313,7 +316,7 @@ WebApiController::Response WebApiController::listFeatures( const Request& reques
 		return checkResponse;
 	}
 
-	const auto& features = m_featureManager.features(); // clazy:exclude=inefficient-qlist
+	const auto& features = VeyonCore::featureManager().features(); // clazy:exclude=inefficient-qlist
 	const auto activeFeatures = lookupConnection( request )->controlInterface()->activeFeatures();
 
 	QVariantList featureList; // clazy:exclude=inefficient-qlist
@@ -353,7 +356,7 @@ WebApiController::Response WebApiController::setFeatureStatus( const Request& re
 																	 : FeatureProviderInterface::Operation::Stop;
 	const auto arguments = request.data[k2s(Key::Arguments)].toMap();
 
-	m_featureManager.controlFeature( feature, operation, arguments, { connection->controlInterface() } );
+	VeyonCore::featureManager().controlFeature( feature, operation, arguments, { connection->controlInterface() } );
 
 	return {};
 }
@@ -423,6 +426,7 @@ QString WebApiController::errorString( WebApiController::Error error )
 	case Error::UnsupportedImageFormat: return QStringLiteral("Unsupported image format");
 	case Error::FramebufferNotAvailable: return QStringLiteral("Framebuffer not yet available");
 	case Error::FramebufferEncodingError: return QStringLiteral("Framebuffer encoding error");
+	case Error::ProtocolMismatch: return QStringLiteral("Protocol mismatch error");
 	}
 
 	return {};
@@ -442,18 +446,38 @@ void WebApiController::removeConnection( QUuid connectionUuid )
 
 
 
+QVariant WebApiController::lookupHeaderField(const Request& request, const QString& fieldName)
+{
+	if( request.headers.contains(fieldName) )
+	{
+		return request.headers[fieldName];
+	}
+
+	for( auto it = request.headers.constBegin(), end = request.headers.constEnd(); it != end; ++it )
+	{
+		if( it.key().compare( fieldName, Qt::CaseInsensitive ) == 0 )
+		{
+			return it.value();
+		}
+	}
+
+	return {};
+}
+
+
+
 WebApiController::LockingConnectionPointer WebApiController::lookupConnection( const Request& request )
 {
 	QReadLocker connectionsReadLocker{&m_connectionsLock};
 
-	return m_connections.value( request.headers[connectionUidHeaderFieldName()].toUuid() );
+	return m_connections.value( lookupHeaderField(request.headers, connectionUidHeaderFieldName()).toUuid() );
 }
 
 
 
 WebApiController::Response WebApiController::checkConnection( const Request& request )
 {
-	const auto connectionUuid = QUuid( request.headers[connectionUidHeaderFieldName()].toString() );
+	const auto connectionUuid = lookupHeaderField(request.headers, connectionUidHeaderFieldName()).toUuid();
 
 	return runInMainThread<WebApiController::Response>( [=]() -> WebApiController::Response {
 		m_connectionsLock.lockForRead();
@@ -482,7 +506,7 @@ WebApiController::Response WebApiController::checkConnection( const Request& req
 
 WebApiController::Response WebApiController::checkFeature( const QString& featureUid )
 {
-	if( m_featureManager.feature( featureUid ).isValid() == false )
+	if( VeyonCore::featureManager().feature( featureUid ).isValid() == false )
 	{
 		return Error::InvalidFeature;
 	}
