@@ -1,7 +1,7 @@
 /*
  * VeyonWorker.cpp - basic implementation of Veyon Worker
  *
- * Copyright (c) 2018-2021 Tobias Junghans <tobydox@veyon.io>
+ * Copyright (c) 2018-2022 Tobias Junghans <tobydox@veyon.io>
  *
  * This file is part of Veyon - https://veyon.io
  *
@@ -24,22 +24,22 @@
 
 #include <QCoreApplication>
 
+#include "FeatureManager.h"
 #include "FeatureWorkerManagerConnection.h"
 #include "VeyonConfiguration.h"
 #include "VeyonWorker.h"
 
 
-VeyonWorker::VeyonWorker( const QString& featureUid, QObject* parent ) :
+VeyonWorker::VeyonWorker( QUuid featureUid, QObject* parent ) :
 	QObject( parent ),
 	m_core( QCoreApplication::instance(),
 			VeyonCore::Component::Worker,
 			QStringLiteral( "FeatureWorker-" ) + VeyonCore::formattedUuid( featureUid ) ),
-	m_featureManager(),
 	m_workerManagerConnection( nullptr )
 {
 	const Feature* workerFeature = nullptr;
 
-	for( const auto& feature : m_featureManager.features() )
+	for( const auto& feature : VeyonCore::featureManager().features() )
 	{
 		if( feature.uid() == featureUid )
 		{
@@ -52,12 +52,12 @@ VeyonWorker::VeyonWorker( const QString& featureUid, QObject* parent ) :
 		qFatal( "Could not find specified feature" );
 	}
 
-	if( m_core.config().disabledFeatures().contains( featureUid ) )
+	if( m_core.config().disabledFeatures().contains( featureUid.toString() ) )
 	{
 		qFatal( "Specified feature is disabled by configuration!" );
 	}
 
-	m_workerManagerConnection = new FeatureWorkerManagerConnection( *this, m_featureManager, featureUid, this );
+	m_workerManagerConnection = new FeatureWorkerManagerConnection( *this, featureUid, this );
 
 	vInfo() << "Running worker for feature" << workerFeature->name();
 }
